@@ -1,3 +1,4 @@
+
 #ifdef FLOW_ENGINE
 
 #include "def_types.h"
@@ -12,8 +13,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include "Network.h"
-
+#include "Network.hpp"
 
 
 #define FAST
@@ -28,49 +28,32 @@ int F1=0, F2=0, Re1=0, Re2=0;
 int fictious_vertex, real_vertex;
 bool facet_detected = false;
 
-const double FAR = 500;
+const double FAR = 5000;
 
 using namespace std;
-using namespace boost;
+// using namespace boost;
 namespace CGT {
-  
+
 Network::~Network(){}
 
 Network::Network(){}
 
-double Network::Volume_Pore ( Cell_handle cell)
-{
-	RTriangulation& Tri = T[currentTes].Triangulation();
-	Real v;
-	switch (cell->info().fictious()) {
-                        case(0) :
-				v=std::abs(Tri.tetrahedron(cell).volume());
-				for (int k=0;k<4;k++) v-= spherical_triangle_volume(cell->vertex(permut4[k][0])->point(),
-					cell->vertex(permut4[k][1])->point(),
-					cell->vertex(permut4[k][2])->point(),
-					cell->vertex(permut4[k][3])->point());
-				return v;
-                        case(1) : return 0;
-                        case(2) : return 0;
-			case(3) : return 0;}
-}
-
 int Network::Detect_facet_fictious_vertices (Cell_handle& cell, int& j)
 {
 	fictious_vertex = 0;
 	int real_vertex=0;
 	Sphere v [3];
         Vertex_handle W [3];
-	
+
         for (int kk=0; kk<3; kk++) {
                 W[kk] = cell->vertex(facetVertices[j][kk]);
                 v[kk] = cell->vertex(facetVertices[j][kk])->point();
                 if (W[kk]->info().isFictious) {
-                        if (fictious_vertex==0) {F1=facetVertices[j][kk];facetF1=kk;} else 
+                        if (fictious_vertex==0) {F1=facetVertices[j][kk];facetF1=kk;} else
 			{F2 = facetVertices[j][kk];facetF2=kk;}
                         fictious_vertex +=1;
                 } else {
-                        if (real_vertex==0) {Re1=facetVertices[j][kk];facetRe1=kk;} else if (real_vertex==1) 
+                        if (real_vertex==0) {Re1=facetVertices[j][kk];facetRe1=kk;} else if (real_vertex==1)
 			{Re2=facetVertices[j][kk];facetRe2=kk;} else if (real_vertex==2)
 			{Re2=facetVertices[j][kk];facetRe3=kk;}
                         real_vertex+=1;}}
@@ -82,58 +65,61 @@ double Network::Volume_Pore_VoronoiFraction (Cell_handle& cell, int& j)
 {
   Point& p1 = cell->info();
   Point& p2 = cell->neighbor(j)->info();
-  
   fictious_vertex = Detect_facet_fictious_vertices (cell,j);
-  
   Sphere v [3];
   Vertex_handle W [3];
   for (int kk=0; kk<3; kk++) {W[kk] = cell->vertex(facetVertices[j][kk]);v[kk] = cell->vertex(facetVertices[j][kk])->point();}
-
   switch (fictious_vertex) {
     case (0) : {
 		Vertex_handle& SV1 = W[0];
                 Vertex_handle& SV2 = W[1];
                 Vertex_handle& SV3 = W[2];
-		
+
                 cell->info().facetSurfaces[j]=0.5*CGAL::cross_product(SV1->point()-SV3->point(),SV2->point()-SV3->point());
                 if (cell->info().facetSurfaces[j]*(p2-p1) > 0) cell->info().facetSurfaces[j] = -1.0*cell->info().facetSurfaces[j];
                 Real Vtot = abs(ONE_THIRD*cell->info().facetSurfaces[j]*(p1-p2));
 		Vtotalissimo += Vtot;
-
+		
                 double Vsolid1=0, Vsolid2=0;
                 for (int i=0;i<3;i++) {
                 Vsolid1 += spherical_triangle_volume(v[permut3[i][0]],v[permut3[i][1]],p1,p2);
                 Vsolid2 += spherical_triangle_volume(v[permut3[i][0]],v[permut3[i][2]],p1,p2);}
-                
+
 		Vsolid_tot += Vsolid1 + Vsolid2;
 		Vporale += Vtot - (Vsolid1 + Vsolid2);
+		
+		bool border=false;
+		for (int i=0;i<4;i++){
+		  if (cell->neighbor(i)->info().fictious()!=0) border=true;}
+		if (!border) {V_porale_porosity += Vtot - (Vsolid1 + Vsolid2);
+		    V_totale_porosity += Vtot;}
 
-		/**Vpore**/ return Vtot - (Vsolid1 + Vsolid2); 
+		/**Vpore**/ return Vtot - (Vsolid1 + Vsolid2);
     }; break;
     case (1) : {return volume_single_fictious_pore(cell->vertex(facetVertices[j][facetF1]), cell->vertex(facetVertices[j][facetRe1]), cell->vertex(facetVertices[j][facetRe2]), p1,p2, cell->info().facetSurfaces[j]);}; break;
     case (2) : {return volume_double_fictious_pore(cell->vertex(facetVertices[j][facetF1]), cell->vertex(facetVertices[j][facetF2]), cell->vertex(facetVertices[j][facetRe1]), p1,p2, cell->info().facetSurfaces[j]);}; break;
-    }
+    default : return 0;}
 }
 
 double Network::volume_single_fictious_pore(const Vertex_handle& SV1, const Vertex_handle& SV2, const Vertex_handle& SV3, const Point& PV1,  const Point& PV2, Vecteur& facetSurface)
 {
         double A [3], B[3];
-	
-        Boundary &bi1 =  boundaries [SV1->info().id()];
 
+        Boundary &bi1 =  boundaries [SV1->info().id()];
+	
         for (int m=0;m<3;m++) {A[m]= (SV2->point())[m];}
         for (int m=0;m<3;m++) {B[m]= (SV3->point())[m];}
-
+        
         A[bi1.coordinate]=bi1.p[bi1.coordinate];
         B[bi1.coordinate]=bi1.p[bi1.coordinate];
-
+	
         Point AA(A[0],A[1],A[2]);
         Point BB(B[0],B[1],B[2]);
         facetSurface = surface_single_fictious_facet(SV1,SV2,SV3);
-        if (facetSurface*(PV2-PV1) > 0) facetSurface = -1.0*facetSurface;
+	if (facetSurface*(PV2-PV1) > 0) facetSurface = -1.0*facetSurface;
         Real Vtot=ONE_THIRD*abs(facetSurface*(PV1-PV2));
 	Vtotalissimo += Vtot;
-
+	
         Sphere A1(AA, 0);
         Sphere B1(BB, 0);
         Sphere& SW2 = SV2->point();
@@ -141,10 +127,10 @@ double Network::volume_single_fictious_pore(const Vertex_handle& SV1, const Vert
 
         Real Vsolid1 = spherical_triangle_volume(SW2, AA, PV1, PV2)+spherical_triangle_volume(SW2, SW3, PV1, PV2);
         Real Vsolid2 = spherical_triangle_volume(SW3, BB, PV1, PV2)+spherical_triangle_volume(SW3, SW2, PV1, PV2);
-
+	
 	Vsolid_tot += Vsolid1 + Vsolid2;
 	Vporale += Vtot - (Vsolid1 + Vsolid2);
-        
+
         return (Vtot - (Vsolid1 + Vsolid2));
 }
 
@@ -155,20 +141,20 @@ double Network::volume_double_fictious_pore(const Vertex_handle& SV1, const Vert
 	Boundary &bi1 =  boundary(SV1->info().id());
         Boundary &bi2 =  boundary(SV2->info().id());
         for (int m=0;m<3;m++) {A[m]=B[m]= SV3->point()[m];}
-        
+
         A[bi1.coordinate]=bi1.p[bi1.coordinate];
         B[bi2.coordinate]=bi2.p[bi2.coordinate];
         Point AA(A[0],A[1],A[2]);
         Point BB(B[0],B[1],B[2]);
-	
+
         facetSurface = CGAL::cross_product(SV3->point()-AA,SV3->point()-BB);
         if (facetSurface*(PV2-PV1) > 0) facetSurface = -1.0*facetSurface;
         Real Vtot = abs(facetSurface*(PV1-PV2))*ONE_THIRD;
 	Vtotalissimo += Vtot;
-  
+
         Real Vsolid1 = spherical_triangle_volume(SV3->point(), AA, PV1, PV2);
         Real Vsolid2 = spherical_triangle_volume(SV3->point(), BB, PV1, PV2);
-	
+
 	Vporale += (Vtot - Vsolid1 - Vsolid2);
 	Vsolid_tot += Vsolid1 + Vsolid2;
 
@@ -179,7 +165,6 @@ double Network::spherical_triangle_volume(const Sphere& ST1, const Point& PT1, c
 {
         double rayon = sqrt(ST1.weight());
         if (rayon == 0.0) return 0.0;
-
         return ((ONE_THIRD * rayon) * (fast_spherical_triangle_area(ST1, PT1, PT2, PT3))) ;
 }
 
@@ -189,7 +174,6 @@ double Network::fast_spherical_triangle_area(const Sphere& STA1, const Point& ST
 #ifndef FAST
         return spherical_triangle_area(STA1, STA2, STA3, PTA1);
 #endif
-
         double rayon2 = STA1.weight();
         if (rayon2 == 0.0) return 0.0;
         return rayon2 * fast_solid_angle(STA1,STA2,STA3,PTA1);
@@ -264,27 +248,27 @@ Real Network::fast_solid_angle(const Point& STA1, const Point& PTA1, const Point
 double Network::Surface_Solid_Pore(Cell_handle cell, int j, bool SLIP_ON_LATERALS)
 {
   if (!facet_detected) fictious_vertex=Detect_facet_fictious_vertices(cell,j);
-  
-  RTriangulation& Tri = T[currentTes].Triangulation();
+
+//   RTriangulation& Tri = T[currentTes].Triangulation();
   Point& p1 = cell->info();
   Point& p2 = cell->neighbor(j)->info();
-  
+
   double Ssolid = 0;
   double Ssolid1= 0, Ssolid1n= 0, Ssolid2= 0, Ssolid2n= 0, Ssolid3= 0, Ssolid3n= 0;
-  
+
   Sphere v [3];
   Vertex_handle W [3];
-  
+
   for (int kk=0; kk<3; kk++) {
 	  W[kk] = cell->vertex(facetVertices[j][kk]);
 	  v[kk] = cell->vertex(facetVertices[j][kk])->point();}
-  
+
   switch (fictious_vertex) {
     case (0) : {
 		Vertex_handle& SV1 = W[0];
                 Vertex_handle& SV2 = W[1];
                 Vertex_handle& SV3 = W[2];
-		
+
 		Ssolid1 = fast_spherical_triangle_area(SV1->point(), SV2->point(), p1, p2);
                 Ssolid1n = fast_spherical_triangle_area(SV1->point(), SV3->point(), p1, p2);
                 cell->info().solidSurfaces[j][0]=Ssolid1+Ssolid1n;
@@ -300,7 +284,7 @@ double Network::Surface_Solid_Pore(Cell_handle cell, int j, bool SLIP_ON_LATERAL
 		Vertex_handle SV1 = cell->vertex(facetVertices[j][facetF1]);
 		Vertex_handle SV2 = cell->vertex(facetVertices[j][facetRe1]);
 		Vertex_handle SV3 = cell->vertex(facetVertices[j][facetRe2]);
-		
+
 		Boundary &bi1 =  boundary(SV1->info().id());
                 Ssolid1 = 0;
 		if (bi1.flowCondition && ! SLIP_ON_LATERALS) {
@@ -315,13 +299,13 @@ double Network::Surface_Solid_Pore(Cell_handle cell, int j, bool SLIP_ON_LATERAL
                 cell->info().solidSurfaces[j][facetRe2]=Ssolid3+Ssolid3n;
     }; break;
     case (2) : {
-      
+
 		double A [3], B[3], C[3];
-		
+
 		Vertex_handle SV1 = cell->vertex(facetVertices[j][facetF1]);
 		Vertex_handle SV2 = cell->vertex(facetVertices[j][facetF2]);
 		Vertex_handle SV3 = cell->vertex(facetVertices[j][facetRe1]);
-		
+
 		Boundary &bi1 =  boundary(SV1->info().id());
                 Boundary &bi2 =  boundary(SV2->info().id());
                 for (int m=0;m<3;m++) {
@@ -357,12 +341,12 @@ double Network::Surface_Solid_Pore(Cell_handle cell, int j, bool SLIP_ON_LATERAL
                 } else cell->info().solidSurfaces[j][facetF2]=0;
     }; break;
     }
-    
+
     Ssolid = Ssolid1+Ssolid1n+Ssolid2+Ssolid2n+Ssolid3+Ssolid3n;
-    
+
     cell->info().solidSurfaces[j][3]=1.0/Ssolid;
     Ssolid_tot += Ssolid;
-    
+
     return Ssolid;
 
 }
@@ -385,7 +369,7 @@ Vecteur Network::surface_single_fictious_facet(Vertex_handle fSV1, Vertex_handle
         const Boundary &bi1 =  boundary(fSV1->info().id());
 //  const Boundary &bi2 = boundary ( fSV2->info().id() );
         Vecteur mean_height = (bi1.p[bi1.coordinate]-0.5*(SV3->point()[bi1.coordinate]+SV2->point()[bi1.coordinate]))*bi1.normal;
-	
+
         return CGAL::cross_product(mean_height,SV3->point()-SV2->point());
 }
 
@@ -438,87 +422,91 @@ void Network::AddBoundingPlanes()
 {
 	Tesselation& Tes = T[currentTes];
 	
-	y_min_id = Tes.Max_id() + 1;
+	//FIXME: Id's order in boundsIds is done according to the enumerotation of boundaries from TXStressController.hpp, line 31. DON'T CHANGE IT!
+	y_min_id = Tes.Max_id() + 2;
         boundsIds[0]=&y_min_id;
-        y_max_id = Tes.Max_id() + 2;
+        y_max_id = Tes.Max_id() + 3;
         boundsIds[1]=&y_max_id;
-        x_min_id = Tes.Max_id() + 3;
+        x_min_id = Tes.Max_id() + 0;
         boundsIds[2]=&x_min_id;
-        x_max_id = Tes.Max_id() + 4;
+        x_max_id = Tes.Max_id() + 1;
         boundsIds[3]=&x_max_id;
         z_min_id = Tes.Max_id() + 5;
-        boundsIds[4]=&z_min_id;
+        boundsIds[4]=&z_max_id;
         z_max_id = Tes.Max_id() + 6;
-        boundsIds[5]=&z_max_id;
-	 
+        boundsIds[5]=&z_min_id;
+	
+	Corner_min = Point(x_min, y_min, z_min);
+	Corner_max = Point(x_max, y_max, z_max);
+	
 	id_offset = Tes.Max_id() +1;//so that boundaries[vertex->id - offset] gives the ordered boundaries (also see function Boundary& boundary(int b))
 	
-	AddBoundingPlanes(true);
+	AddBoundingPlane (true, Vecteur(0,1,0) , y_min_id);
+	AddBoundingPlane (true, Vecteur(0,-1,0) , y_max_id);
+	AddBoundingPlane (true, Vecteur(-1,0,0) , x_max_id);
+	AddBoundingPlane (true, Vecteur(1,0,0) , x_min_id);
+	AddBoundingPlane (true, Vecteur(0,0,1) , z_min_id);
+	AddBoundingPlane (true, Vecteur(0,0,-1) , z_max_id);
+
+// 	AddBoundingPlanes(true);
 }
 
-void Network::AddBoundingPlanes(bool yade)
+void Network::AddBoundingPlane (bool yade, Vecteur Normal, int id_wall)
 {
-        Tesselation& Tes = T[currentTes];
-        Corner_min = Point(x_min, y_min, z_min);
-        Corner_max = Point(x_max, y_max, z_max);
+	  Tesselation& Tes = T[currentTes];
+	  
+	  int Coordinate = abs(Normal[0])*0 + abs(Normal[1])*1 + abs(Normal[2])*2;
+	  
+	  double pivot = Normal[Coordinate]<0 ? 
+	  Corner_max.x()*abs(Normal[0])+Corner_max.y()*abs(Normal[1])+Corner_max.z()*abs(Normal[2]) : Corner_min.x()*abs(Normal[0])+Corner_min.y()*abs(Normal[1])+Corner_min.z()*abs(Normal[2]);
+	
+	  Tes.insert(0.5*(Corner_min.x() +Corner_max.x())*(1-abs(Normal[0]))+(pivot-Normal[0]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[0]),
+		     0.5*(Corner_max.y() +Corner_min.y())*(1-abs(Normal[1]))+(pivot-Normal[1]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[1]),
+		     0.5*(Corner_max.z() +Corner_min.z())*(1-abs(Normal[2]))+(pivot-Normal[2]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[2]),
+		     FAR*(Corner_max.y()-Corner_min.y()), id_wall, true);
 
-        boundsIds[0]= &y_min_id;
-        boundsIds[1]= &y_max_id;
-        boundsIds[2]= &x_min_id;
-        boundsIds[3]= &x_max_id;
-        boundsIds[4]= &z_min_id;
-        boundsIds[5]= &z_max_id;
+	  if (Normal[Coordinate]<0) boundaries[id_wall-id_offset].p = Corner_max;
+	  else boundaries[id_wall-id_offset].p = Corner_min;
+	  
+	  boundaries[id_wall-id_offset].normal = Normal;
+	  boundaries[id_wall-id_offset].coordinate = Coordinate;
+	  
+          boundaries[id_wall-id_offset].flowCondition = 1;
+          boundaries[id_wall-id_offset].value = 0;
+	  
+	  if(DEBUG_OUT) cout << "A boundary -max/min-has been created. ID = " << id_wall << " position = " << 0.5*(Corner_min.x() +Corner_max.x())*(1-abs(Normal[0]))+(pivot-Normal[0]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[0]) << " , " << 0.5*(Corner_max.y() +Corner_min.y())*(1-abs(Normal[1]))+(pivot-Normal[1]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[1]) << " , " << 0.5*(Corner_max.z() +Corner_min.z())*(1-abs(Normal[2]))+(pivot-Normal[2]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[2])  << ". Radius = " << FAR*(Corner_max.y()-Corner_min.y()) << endl;
+}
 
-        Tes.insert(0.5*(Corner_min.x() +Corner_max.x()), Corner_min.y()-FAR*(Corner_max.x()-Corner_min.x()), 0.5*(Corner_max.z()-Corner_min.z()), FAR*(Corner_max.x()-Corner_min.x()), y_min_id, true);
-        boundaries[y_min_id-id_offset].p = Corner_min;
-        boundaries[y_min_id-id_offset].normal = Vecteur(0,1,0);
-        boundaries[y_min_id-id_offset].coordinate = 1;
-        cout << "Bottom boundary has been created. ID = " << y_min_id << endl;
-
-        Tes.insert(0.5*(Corner_min.x() +Corner_max.x()), Corner_max.y() +FAR*(Corner_max.x()-Corner_min.x()), 0.5*(Corner_max.z()-Corner_min.z()), FAR*(Corner_max.x()-Corner_min.x()), y_max_id, true);
-        boundaries[y_max_id-id_offset].p = Corner_max;
-        boundaries[y_max_id-id_offset].normal = Vecteur(0,-1,0);
-        boundaries[y_max_id-id_offset].coordinate = 1;
-        cout << "Top boundary has been created. ID = " << y_max_id << endl;
-
-        Tes.insert(Corner_min.x()-FAR*(Corner_max.y()-Corner_min.y()), 0.5*(Corner_max.y()-Corner_min.y()), 0.5*(Corner_max.z()-Corner_min.z()), FAR*(Corner_max.y()-Corner_min.y()), x_min_id, true);
-        boundaries[x_min_id-id_offset].p = Corner_min;
-        boundaries[x_min_id-id_offset].normal = Vecteur(1,0,0);
-        boundaries[x_min_id-id_offset].coordinate = 0;
-        cout << "Left boundary has been created. ID = " << x_min_id << endl;
-
-        Tes.insert(Corner_max.x() +FAR*(Corner_max.y()-Corner_min.y()), 0.5*(Corner_max.y()-Corner_min.y()), 0.5*(Corner_max.z()-Corner_min.z()), FAR*(Corner_max.y()-Corner_min.y()), x_max_id, true);
-        boundaries[x_max_id-id_offset].p = Corner_max;
-        boundaries[x_max_id-id_offset].normal = Vecteur(-1,0,0);
-        boundaries[x_max_id-id_offset].coordinate = 0;
-        cout << "Right boundary has been created. ID = " << x_max_id << endl;
-
-        Tes.insert(0.5*(Corner_min.x() +Corner_max.x()), 0.5*(Corner_max.y()-Corner_min.y()), Corner_min.z()-FAR*(Corner_max.y()-Corner_min.y()), FAR*(Corner_max.y()-Corner_min.y()), z_min_id, true);
-        boundaries[z_min_id-id_offset].p = Corner_min;
-        boundaries[z_min_id-id_offset].normal = Vecteur(0,0,1);
-        boundaries[z_min_id-id_offset].coordinate = 2;
-        cout << "Front boundary has been created. ID = " << z_min_id << endl;
-
-        Tes.insert(0.5*(Corner_min.x() +Corner_max.x()), 0.5*(Corner_max.y()-Corner_min.y()), Corner_max.z() +FAR*(Corner_max.y()-Corner_min.y()), FAR*(Corner_max.y()-Corner_min.y()), z_max_id, true);
-        boundaries[z_max_id-id_offset].p = Corner_max;
-        boundaries[z_max_id-id_offset].normal = Vecteur(0,0,-1);
-        boundaries[z_max_id-id_offset].coordinate = 2;
-        cout << "Back boundary has been created. ID = " << z_max_id << endl;
-
-        for (int k=0;k<6;k++) {
-                boundaries[k].flowCondition = 1;
-                boundaries[k].value = 0;
-        }
+void Network::AddBoundingPlane (Real center[3], double thickness, Vecteur Normal, int id_wall )
+{
+	  Tesselation& Tes = T[currentTes];
+	  
+	  int Coordinate = abs(Normal[0])*0 + abs(Normal[1])*1 + abs(Normal[2])*2;
+	  
+	  Tes.insert((center[0]+Normal[0]*thickness/2)*(1-abs(Normal[0])) + (center[0]+Normal[0]*thickness/2-Normal[0]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[0]),
+		     (center[1]+Normal[1]*thickness/2)*(1-abs(Normal[1])) + (center[1]+Normal[1]*thickness/2-Normal[1]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[1]),
+		     (center[2]+Normal[2]*thickness/2)*(1-abs(Normal[2])) + (center[2]+Normal[2]*thickness/2-Normal[2]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[2]),
+		     FAR*(Corner_max.y()-Corner_min.y()), id_wall, true);
+	  
+ 	  Point P (center[0],center[1],center[2]);
+	  boundaries[id_wall-id_offset].p = P;
+	  boundaries[id_wall-id_offset].normal = Normal;
+	  boundaries[id_wall-id_offset].coordinate = Coordinate;
+	  
+          boundaries[id_wall-id_offset].flowCondition = 1;
+          boundaries[id_wall-id_offset].value = 0;
+	  
+	  if(DEBUG_OUT) cout << "A boundary -center/thick- has been created. ID = " << id_wall << " position = " << (center[0]+Normal[0]*thickness/2)*(1-abs(Normal[0])) + (center[0]+Normal[0]*thickness/2-Normal[0]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[0]) << " , " << (center[1]+Normal[1]*thickness/2)*(1-abs(Normal[1])) + (center[1]+Normal[1]*thickness/2-Normal[1]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[1]) << " , " <<  (center[2]+Normal[2]*thickness/2)*(1-abs(Normal[2])) + (center[2]+Normal[2]*thickness/2-Normal[2]*FAR*(Corner_max.y()-Corner_min.y()))*abs(Normal[2]) << ". Radius = " << FAR*(Corner_max.y()-Corner_min.y()) << endl;
 }
 
 void Network::Define_fictious_cells()
 {
 	RTriangulation& Tri = T[currentTes].Triangulation();
-  
-	Finite_cells_iterator cell_end = Tri.finite_cells_end();     
+
+	Finite_cells_iterator cell_end = Tri.finite_cells_end();
 	for (Finite_cells_iterator cell = Tri.finite_cells_begin(); cell != cell_end; cell++) {
 	  cell->info().fictious()=0;}
-  
+
 	for (int bound=0; bound<6;bound++) {
                 int& id = *boundsIds[bound];
                 Tesselation::Vector_Cell tmp_cells;
@@ -531,46 +519,8 @@ void Network::Define_fictious_cells()
 		  (cell->info().fictious())+=1;
 		}
 	}
-}
-
-void Network::DisplayStatistics()
-{
-	RTriangulation& Tri = T[currentTes].Triangulation();
-        int Zero =0, Inside=0, Fictious=0;
-        Finite_cells_iterator cell_end = Tri.finite_cells_end();
-        for (Finite_cells_iterator cell = Tri.finite_cells_begin(); cell != cell_end; cell++) {
-                int zeros =0;
-                for (int j=0; j!=4; j++) {
-                        if ((cell->info().k_norm())[j]==0) {
-                                zeros+=1;
-                        }
-                }
-                if (zeros==4) {
-                        Zero+=1;
-                }
-                if (!cell->info().fictious()) {
-                        Inside+=1;
-                } else {
-                        Fictious+=1;
-                }
-        }
-        cout << "zeros = " << Zero << endl;
-        int fict=0, real=0;
-        for (Finite_vertices_iterator v = Tri.finite_vertices_begin(); v != Tri.finite_vertices_end(); ++v) {
-                if (v->info().isFictious) fict+=1;
-                else real+=1;
-        }
-        long Vertices = Tri.number_of_vertices();
-        cout << "There are " << Vertices << " vertices, dont " << fict << " fictious et " << real << " reeeeeel" << std::endl;
-        long Cells = Tri.number_of_finite_cells();
-        cout << "There are " << Cells << " cells " << std::endl;
-        long Facets = Tri.number_of_finite_facets();
-        cout << "There are " << Facets << " facets " << std::endl;
-        cout << "There are " << Inside << " cells INSIDE." << endl;
-        cout << "There are " << Fictious << " cells FICTIOUS." << endl;
 	
-	vtk_infinite_vertices = fict;
-	vtk_infinite_cells = Fictious;
+	if(DEBUG_OUT) cout << "Fictious cell defined" << endl;
 }
 
 // double Network::spherical_triangle_area ( Sphere STA1, Sphere STA2, Sphere STA3, Point PTA1 )
@@ -605,4 +555,4 @@ void Network::DisplayStatistics()
 // }
 } //namespace CGT
 
-#endif //FLOW_ENGINE
\ No newline at end of file
+#endif //FLOW_ENGINE
