@@ -1,10 +1,10 @@
 // 2007,2008 © Václav Šmilauer <eudoxos@arcig.cz> 
 #include"ConcretePM.hpp"
 #include<yade/core/Scene.hpp>
-#include<yade/pkg-dem/DemXDofGeom.hpp>
-#include<yade/pkg-dem/Shop.hpp>
+#include<yade/pkg/dem/DemXDofGeom.hpp>
+#include<yade/pkg/dem/Shop.hpp>
 
-YADE_PLUGIN((CpmState)(CpmMat)(Ip2_CpmMat_CpmMat_CpmPhys)(CpmPhys)(Law2_Dem3DofGeom_CpmPhys_Cpm)(Law2_ScGeom_CpmPhys_Cpm)
+YADE_PLUGIN((CpmState)(CpmMat)(Ip2_CpmMat_CpmMat_CpmPhys)(CpmPhys)(Law2_Dem3DofGeom_CpmPhys_Cpm)
 	#ifdef YADE_OPENGL
 		(Gl1_CpmPhys)
 	#endif	
@@ -25,23 +25,32 @@ void Ip2_CpmMat_CpmMat_CpmPhys::go(const shared_ptr<Material>& pp1, const shared
 
 	// check unassigned values
 	assert(!isnan(mat1->G_over_E));
+	assert(!isnan(mat2->G_over_E));
 	if(!mat1->neverDamage) {
 		assert(!isnan(mat1->sigmaT));
 		assert(!isnan(mat1->epsCrackOnset));
-		assert(!isnan(mat1->relDuctility));
+		assert(!isnan(mat1->crackOpening) || !isnan(mat1->relDuctility));
 		assert(!isnan(mat1->G_over_E));
 	}
+	if(!mat2->neverDamage) {
+		assert(!isnan(mat2->sigmaT));
+		assert(!isnan(mat2->epsCrackOnset));
+		assert(!isnan(mat2->crackOpening) || !isnan(mat2->relDuctility));
+		assert(!isnan(mat2->G_over_E));
+	}
 
+	cpmPhys->damLaw = mat1->damLaw;
 	// bodies sharing the same material; no averages necessary
 	if(mat1->id>=0 && mat1->id==mat2->id) {
 		cpmPhys->E=mat1->young;
 		cpmPhys->G=mat1->young*mat1->G_over_E;
 		cpmPhys->tanFrictionAngle=tan(mat1->frictionAngle);
 		cpmPhys->undamagedCohesion=mat1->sigmaT;
-		cpmPhys->epsFracture=mat1->relDuctility*mat1->epsCrackOnset;
 		cpmPhys->isCohesive=(cohesiveThresholdIter<0 || scene->iter<cohesiveThresholdIter);
 		#define _CPATTR(a) cpmPhys->a=mat1->a
 			_CPATTR(epsCrackOnset);
+			_CPATTR(crackOpening);
+			_CPATTR(relDuctility);
 			_CPATTR(neverDamage);
 			_CPATTR(dmgTau);
 			_CPATTR(dmgRateExp);
@@ -56,9 +65,10 @@ void Ip2_CpmMat_CpmMat_CpmPhys::go(const shared_ptr<Material>& pp1, const shared
 			cpmPhys->G=.5*(mat1->G_over_E+mat2->G_over_E)*.5*(mat1->young+mat2->young);
 			cpmPhys->tanFrictionAngle=tan(.5*(mat1->frictionAngle+mat2->frictionAngle));
 			cpmPhys->undamagedCohesion=.5*(mat1->sigmaT+mat2->sigmaT);
-			cpmPhys->epsFracture=.5*(mat1->relDuctility+mat2->relDuctility)*.5*(mat1->epsCrackOnset+mat2->epsCrackOnset);
 			cpmPhys->isCohesive=(cohesiveThresholdIter<0 || scene->iter<cohesiveThresholdIter);
 			_AVGATTR(epsCrackOnset);
+			_AVGATTR(crackOpening);
+			_AVGATTR(relDuctility);
 			cpmPhys->neverDamage=(mat1->neverDamage || mat2->neverDamage);
 			_AVGATTR(dmgTau);
 			_AVGATTR(dmgRateExp);
@@ -67,6 +77,7 @@ void Ip2_CpmMat_CpmMat_CpmPhys::go(const shared_ptr<Material>& pp1, const shared
 			_AVGATTR(isoPrestress);
 		#undef _AVGATTR
 	}
+
 	// NOTE: some params are not assigned until in Law2_Dem3DofGeom_CpmPhys_Cpm, since they need geometry as well; those are:
 	// 	crossSection, kn, ks
 }
@@ -148,15 +159,44 @@ void Law2_Dem3DofGeom_CpmPhys_Cpm::go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys
 		BC->crossSection=Mathr::PI*pow(minRad,2);
 		BC->kn=BC->crossSection*BC->E/contGeom->refLength;
 		BC->ks=BC->crossSection*BC->G/contGeom->refLength;
+		BC->epsFracture = isnan(BC->crackOpening)? BC->epsCrackOnset*BC->relDuctility : BC->crackOpening/contGeom->refLength;
 	}
 
 	// shorthands
 		Real& epsN(BC->epsN);
-		Vector3r& epsT(BC->epsT); Real& kappaD(BC->kappaD); Real& epsPlSum(BC->epsPlSum); const Real& E(BC->E); const Real& undamagedCohesion(BC->undamagedCohesion); const Real& tanFrictionAngle(BC->tanFrictionAngle); const Real& G(BC->G); const Real& crossSection(BC->crossSection); const Real& omegaThreshold(Law2_Dem3DofGeom_CpmPhys_Cpm::omegaThreshold); const Real& epsCrackOnset(BC->epsCrackOnset); Real& relResidualStrength(BC->relResidualStrength); const Real& epsFracture(BC->epsFracture); const bool& neverDamage(BC->neverDamage); Real& omega(BC->omega); Real& sigmaN(BC->sigmaN);  Vector3r& sigmaT(BC->sigmaT); Real& Fn(BC->Fn); Vector3r& Fs(BC->Fs); // for python access
-	 const bool& isCohesive(BC->isCohesive);
+		Vector3r& epsT(BC->epsT);
+		Real& kappaD(BC->kappaD);
+		Real& epsPlSum(BC->epsPlSum);
+		const Real& E(BC->E);
+		const Real& undamagedCohesion(BC->undamagedCohesion);
+		const Real& tanFrictionAngle(BC->tanFrictionAngle);
+		const Real& G(BC->G);
+		const Real& crossSection(BC->crossSection);
+		const Real& omegaThreshold(Law2_Dem3DofGeom_CpmPhys_Cpm::omegaThreshold);
+		const Real& epsCrackOnset(BC->epsCrackOnset);
+		Real& relResidualStrength(BC->relResidualStrength);
+		const Real& crackOpening(BC->crackOpening);
+		const Real& relDuctility(BC->relDuctility);
+		const Real& epsFracture(BC->epsFracture);
+		const int& damLaw(BC->damLaw);
+		const bool& neverDamage(BC->neverDamage);
+		Real& omega(BC->omega);
+		Real& sigmaN(BC->sigmaN);
+		Vector3r& sigmaT(BC->sigmaT);
+		Real& Fn(BC->Fn);
+		Vector3r& Fs(BC->Fs); // for python access
+		const bool& isCohesive(BC->isCohesive);
 
 	#ifdef CPM_MATERIAL_MODEL
-		Real& epsNPl(BC->epsNPl); const Real& dt=scene->dt; const Real& dmgTau(BC->dmgTau); const Real& plTau(BC->plTau);const Real& yieldLogSpeed(Law2_Dem3DofGeom_CpmPhys_Cpm::yieldLogSpeed); const int& yieldSurfType(Law2_Dem3DofGeom_CpmPhys_Cpm::yieldSurfType); const Real& yieldEllipseShift(Law2_Dem3DofGeom_CpmPhys_Cpm::yieldEllipseShift); const Real& epsSoft(Law2_Dem3DofGeom_CpmPhys_Cpm::epsSoft); const Real& relKnSoft(Law2_Dem3DofGeom_CpmPhys_Cpm::relKnSoft); 
+		Real& epsNPl(BC->epsNPl);
+		const Real& dt=scene->dt;
+		const Real& dmgTau(BC->dmgTau);
+		const Real& plTau(BC->plTau);
+		const Real& yieldLogSpeed(Law2_Dem3DofGeom_CpmPhys_Cpm::yieldLogSpeed);
+		const int& yieldSurfType(Law2_Dem3DofGeom_CpmPhys_Cpm::yieldSurfType);
+		const Real& yieldEllipseShift(Law2_Dem3DofGeom_CpmPhys_Cpm::yieldEllipseShift);
+		const Real& epsSoft(Law2_Dem3DofGeom_CpmPhys_Cpm::epsSoft);
+		const Real& relKnSoft(Law2_Dem3DofGeom_CpmPhys_Cpm::relKnSoft); 
 	#endif
 
 	epsN=contGeom->strainN(); epsT=contGeom->strainT();
@@ -189,7 +229,8 @@ void Law2_Dem3DofGeom_CpmPhys_Cpm::go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys
 		epsN+=BC->isoPrestress/E;
 		// very simplified version of the constitutive law
 		kappaD=max(max(0.,epsN),kappaD); // internal variable, max positive strain (non-decreasing)
-		omega=isCohesive?funcG(kappaD,epsCrackOnset,epsFracture,neverDamage):1.; // damage variable (non-decreasing, as funcG is also non-decreasing)
+		//Real epsFracture = crackOpening/contGeom->refLength;
+		omega=isCohesive?funcG(kappaD,epsCrackOnset,epsFracture,neverDamage,damLaw):1.; // damage variable (non-decreasing, as funcG is also non-decreasing)
 		sigmaN=(1-(epsN>0?omega:0))*E*epsN; // damage taken in account in tension only
 		sigmaT=G*epsT; // trial stress
 		Real yieldSigmaT=max((Real)0.,undamagedCohesion*(1-omega)-sigmaN*tanFrictionAngle); // Mohr-Coulomb law with damage
@@ -203,7 +244,7 @@ void Law2_Dem3DofGeom_CpmPhys_Cpm::go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys
 	#endif
 	sigmaN-=BC->isoPrestress;
 
-	NNAN(kappaD); NNAN(epsCrackOnset); NNAN(epsFracture); NNAN(omega);
+	NNAN(kappaD); NNAN(epsFracture); NNAN(omega);
 	NNAN(sigmaN); NNANV(sigmaT); NNAN(crossSection);
 
 	// handle broken contacts
@@ -235,7 +276,7 @@ Real Law2_Dem3DofGeom_CpmPhys_Cpm::yieldSigmaTMagnitude(Real sigmaN, Real omega,
 #endif
 }
 
-
+#if 0
 CREATE_LOGGER(Law2_ScGeom_CpmPhys_Cpm);
 void Law2_ScGeom_CpmPhys_Cpm::go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys>& _phys, Interaction* I){
 	ScGeom* geom=static_cast<ScGeom*>(_geom.get());
@@ -303,12 +344,12 @@ void Law2_ScGeom_CpmPhys_Cpm::go(shared_ptr<IGeom>& _geom, shared_ptr<IPhys>& _p
 
 	applyForceAtContactPoint(BC->normalForce+BC->shearForce,geom->contactPoint,I->getId1(),pos1,I->getId2(),pos2);
 }
-
+#endif
 
 #ifdef YADE_OPENGL
 	/********************** Gl1_CpmPhys ****************************/
-	#include<yade/lib-opengl/OpenGLWrapper.hpp>
-	#include<yade/lib-opengl/GLUtils.hpp>
+	#include<yade/lib/opengl/OpenGLWrapper.hpp>
+	#include<yade/lib/opengl/GLUtils.hpp>
 
 	CREATE_LOGGER(Gl1_CpmPhys);
 
@@ -399,25 +440,24 @@ void CpmStateUpdater::update(Scene* _scene){
 		shared_ptr<CpmPhys> phys=dynamic_pointer_cast<CpmPhys>(I->phys);
 		if(!phys) continue;
 		const Body::id_t id1=I->getId1(), id2=I->getId2();
-		GenericSpheresContact* geom=YADE_CAST<GenericSpheresContact*>(I->geom.get());
+		Dem3DofGeom* geom=YADE_CAST<Dem3DofGeom*>(I->geom.get());
+
+		Matrix3r stressTimesV = Matrix3r::Zero();
+		const Vector3r& n= geom->normal;
+		const Real& Fn = phys->Fn;
+		const Vector3r& Fs = phys->Fs;
+		//stressTimesV[i,j] += geom->refLength*(Fn*n[i]*n[j]+0.5*(Fs[i]*n[j]+Fs[j]*n[i]));
+		//stressTimesV += geom->refLength*(Fn*outer(n,n)+.5*(outer(Fs,n)+outer(n,Fs)));
+		stressTimesV += geom->refLength*(Fn*n*n.transpose()+.5*(Fs*n.transpose()+n*Fs.transpose()));
 		
-		Vector3r normalStress=((1./phys->crossSection)*geom->normal.dot(phys->normalForce))*geom->normal;
-		bodyStats[id1].sigma+=normalStress; bodyStats[id2].sigma+=normalStress;
-		Vector3r shearStress;
-		for(int i=0; i<3; i++){
-			int ix1=(i+1)%3,ix2=(i+2)%3;
-			shearStress[i]=geom->normal[ix1]*phys->shearForce[ix1]+geom->normal[ix2]*phys->shearForce[ix2];
-			shearStress[i]/=phys->crossSection;
-		}
-		bodyStats[id1].tau+=shearStress;
-		bodyStats[id2].tau+=shearStress;
+		bodyStats[id1].stressTimesV += stressTimesV;
+		bodyStats[id2].stressTimesV += stressTimesV;
 		bodyStats[id1].nLinks++; bodyStats[id2].nLinks++;
 		
 		if(!phys->isCohesive) continue;
 		bodyStats[id1].nCohLinks++; bodyStats[id1].dmgSum+=(1-phys->relResidualStrength); bodyStats[id1].epsPlSum+=phys->epsPlSum;
 		bodyStats[id2].nCohLinks++; bodyStats[id2].dmgSum+=(1-phys->relResidualStrength); bodyStats[id2].epsPlSum+=phys->epsPlSum;
 		maxOmega=max(maxOmega,phys->omega);
-
 		avgRelResidual+=phys->relResidualStrength;
 		nAvgRelResidual+=1;
 	}
@@ -427,8 +467,8 @@ void CpmStateUpdater::update(Scene* _scene){
 		// add damaged contacts that have already been deleted
 		CpmState* state=dynamic_cast<CpmState*>(B->state.get());
 		if(!state) continue;
-		state->sigma=bodyStats[id].sigma;
-		state->tau=bodyStats[id].tau;
+		state->stressTimesV=bodyStats[id].stressTimesV;
+		//state->tau=bodyStats[id].tau;
 		int cohLinksWhenever=bodyStats[id].nCohLinks+state->numBrokenCohesive;
 		if(cohLinksWhenever>0){
 			state->normDmg=(bodyStats[id].dmgSum+state->numBrokenCohesive)/cohLinksWhenever;
@@ -438,7 +478,7 @@ void CpmStateUpdater::update(Scene* _scene){
 			}
 		}
 		else { state->normDmg=0; state->normEpsPl=0;}
-		B->shape->color=Vector3r(state->normDmg,1-state->normDmg,B->isDynamic()?0:1);
+		B->shape->color=Vector3r(state->normDmg,1-state->normDmg,B->state->blockedDOFs==State::DOF_ALL?0:1);
 		nAvgRelResidual+=0.5*state->numBrokenCohesive; // add half or broken interactions, other body has the other half
 	}
 	avgRelResidual/=nAvgRelResidual;
