@@ -2,10 +2,11 @@
 
 
 #include"HertzMindlin.hpp"
-#include<yade/pkg-dem/ScGeom.hpp>
+#include<yade/pkg/dem/ScGeom.hpp>
 #include<yade/core/Omega.hpp>
 #include<yade/core/Scene.hpp>
 
+
 YADE_PLUGIN(
 	(MindlinPhys)
 	(Ip2_FrictMat_FrictMat_MindlinPhys)
@@ -34,6 +35,7 @@ void Ip2_FrictMat_FrictMat_MindlinPhys::go(const shared_ptr<Material>& b1,const
 	interaction->phys = mindlinPhys;
 	FrictMat* mat1 = YADE_CAST<FrictMat*>(b1.get());
 	FrictMat* mat2 = YADE_CAST<FrictMat*>(b2.get());
+	
 
 	
 	/* from interaction physics */
@@ -46,9 +48,9 @@ void Ip2_FrictMat_FrictMat_MindlinPhys::go(const shared_ptr<Material>& b1,const
 
 
 	/* from interaction geometry */
-	ScGeom* scg = YADE_CAST<ScGeom*>(interaction->geom.get());		
-	Real Da = scg->radius1; 
-	Real Db = scg->radius2; 
+	GenericSpheresContact* scg = YADE_CAST<GenericSpheresContact*>(interaction->geom.get());		
+	Real Da = scg->refR1>0 ? scg->refR1 : scg->refR2; 
+	Real Db = scg->refR2; 
 	Vector3r normal=scg->normal; 
 
 
@@ -59,6 +61,7 @@ void Ip2_FrictMat_FrictMat_MindlinPhys::go(const shared_ptr<Material>& b1,const
 	Real V = (Va+Vb)/2; // average of poisson's ratio
 	Real E = Ea*Eb/((1.-std::pow(Va,2))*Eb+(1.-std::pow(Vb,2))*Ea); // Young modulus
 	Real R = Da*Db/(Da+Db); // equivalent radius
+	Real Rmean = (Da+Db)/2.; // mean radius
 	Real Kno = 4./3.*E*sqrt(R); // coefficient for normal stiffness
 	Real Kso = 2*sqrt(4*R)*G/(2-V); // coefficient for shear stiffness
 	Real frictionAngle = std::min(fa,fb);
@@ -67,32 +70,32 @@ void Ip2_FrictMat_FrictMat_MindlinPhys::go(const shared_ptr<Material>& b1,const
 
 	/* pass values calculated from above to MindlinPhys */
 	mindlinPhys->tangensOfFrictionAngle = std::tan(frictionAngle); 
-	mindlinPhys->prevNormal = scg->normal;
+	//mindlinPhys->prevNormal = scg->normal; // used to compute relative rotation
 	mindlinPhys->kno = Kno; // this is just a coeff
 	mindlinPhys->kso = Kso; // this is just a coeff
 	mindlinPhys->adhesionForce = Adhesion;
+	
+	mindlinPhys->kr = krot;
+	mindlinPhys->ktw = ktwist;
+	mindlinPhys->maxBendPl = eta*Rmean; // does this make sense? why do we take Rmean?
 
 	/* compute viscous coefficients */
 	if(en && betan) throw std::invalid_argument("Ip2_FrictMat_FrictMat_MindlinPhys: only one of en, betan can be specified.");
 	if(es && betas) throw std::invalid_argument("Ip2_FrictMat_FrictMat_MindlinPhys: only one of es, betas can be specified.");
 
-	// en specified, compute betan
-	if(en){
-		Real logE=log((*en)(mat1->id,mat2->id));
-		mindlinPhys->betan=-logE/sqrt(pow(Mathr::PI,2)+pow(logE,2));
+	// en or es specified, just compute alpha, otherwise alpha remains 0
+	if(en || es){
+		Real logE = log((*en)(mat1->id,mat2->id));
+		mindlinPhys->alpha = -sqrt(5/6.)*2*logE/sqrt(pow(logE,2)+pow(Mathr::PI,2))*sqrt(2*E*sqrt(R)); // (see Tsuji, 1992)
 	}
+	
 	// betan specified, use that value directly; otherwise give zero
-	else mindlinPhys->betan=betan ? (*betan)(mat1->id,mat2->id) : 0;
-
-	// same for es, except that when not given, use the value of betan
-	if(es){
-		Real logE=log((*es)(mat1->id,mat2->id));
-		mindlinPhys->betas=-logE/sqrt(pow(Mathr::PI,2)+pow(logE,2));
+	else{	
+		mindlinPhys->betan=betan ? (*betan)(mat1->id,mat2->id) : 0; 
+		mindlinPhys->betas=betas ? (*betas)(mat1->id,mat2->id) : mindlinPhys->betan;
 	}
-	else mindlinPhys->betas=betas ? (*betas)(mat1->id,mat2->id) : mindlinPhys->betan;
 }
 
-
 /* Function to count the number of adhesive contacts in the simulation at each time step */
 Real Law2_ScGeom_MindlinPhys_Mindlin::contactsAdhesive() // It is returning something rather than zero only if includeAdhesion is set to true
 {
@@ -105,6 +108,20 @@ Real Law2_ScGeom_MindlinPhys_Mindlin::contactsAdhesive() // It is returning some
 	return contactsAdhesive;
 }
 
+/* Function which returns the ratio between the number of sliding contacts to the total number at a given time */
+Real Law2_ScGeom_MindlinPhys_Mindlin::ratioSlidingContacts()
+{
+	Real ratio(0); int count(0);
+	FOREACH(const shared_ptr<Interaction>& I, *scene->interactions){
+		if(!I->isReal()) continue;
+		MindlinPhys* phys = dynamic_cast<MindlinPhys*>(I->phys.get());
+		if (phys->isSliding) {ratio+=1;}
+		count++;
+	}  
+	ratio/=count;
+	return ratio;
+}
+
 /* Function to get the NORMAL elastic potential energy of the system */
 Real Law2_ScGeom_MindlinPhys_Mindlin::normElastEnergy()
 {
@@ -121,6 +138,23 @@ Real Law2_ScGeom_MindlinPhys_Mindlin::normElastEnergy()
 	return normEnergy;
 }
 
+/* Function to get the adhesion energy of the system */
+Real Law2_ScGeom_MindlinPhys_Mindlin::adhesionEnergy()
+{
+	Real adhesionEnergy=0;
+	FOREACH(const shared_ptr<Interaction>& I, *scene->interactions){
+		if(!I->isReal()) continue;
+		ScGeom* scg = dynamic_cast<ScGeom*>(I->geom.get());
+		MindlinPhys* phys = dynamic_cast<MindlinPhys*>(I->phys.get());
+		if (phys && includeAdhesion) {
+			Real R=scg->radius1*scg->radius2/(scg->radius1+scg->radius2);
+			Real gammapi=phys->adhesionForce/(4.*R);
+			adhesionEnergy += gammapi*pow(phys->radius,2);} // note that contact radius is calculated if we calculate energy components
+	}
+	return adhesionEnergy;
+}
+
+
 void Law2_ScGeom_MindlinPhys_MindlinDeresiewitz::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys>& ip, Interaction* contact){
 	Body::id_t id1(contact->getId1()), id2(contact->getId2());
 	ScGeom* geom = static_cast<ScGeom*>(ig.get());
@@ -177,8 +211,10 @@ void Law2_ScGeom_MindlinPhys_HertzWithLinearShear::go(shared_ptr<IGeom>& ig, sha
 	Vector3r shearIncrement;
 	if(nonLin>1){
 		State *de1=Body::byId(id1,scene)->state.get(), *de2=Body::byId(id2,scene)->state.get();	
-		Vector3r shift2=scene->isPeriodic ? Vector3r(scene->cell->Hsize*contact->cellDist.cast<Real>()) : Vector3r::Zero();
-		Vector3r shiftVel=scene->isPeriodic ? Vector3r(scene->cell->velGrad*scene->cell->Hsize*contact->cellDist.cast<Real>()) : Vector3r::Zero();
+		Vector3r shiftVel=scene->isPeriodic ? Vector3r(scene->cell->velGrad*scene->cell->hSize*contact->cellDist.cast<Real>()) : Vector3r::Zero();
+		Vector3r shift2 = scene->isPeriodic ? Vector3r(scene->cell->hSize*contact->cellDist.cast<Real>()): Vector3r::Zero();
+		
+		
 		Vector3r incidentV = geom->getIncidentVel(de1, de2, scene->dt, shift2, shiftVel, /*preventGranularRatcheting*/ nonLin>2 );	
 		Vector3r incidentVn = geom->normal.dot(incidentV)*geom->normal; // contact normal velocity
 		Vector3r incidentVs = incidentV-incidentVn; // contact shear velocity
@@ -213,10 +249,12 @@ void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys
 	ScGeom* scg = static_cast<ScGeom*>(ig.get());
 	MindlinPhys* phys = static_cast<MindlinPhys*>(ip.get());	
 
-	const shared_ptr<Body>& b1=Body::byId(id1,scene); // not sure here (I need it because I want to know if the body isDynamic or not)
-	const shared_ptr<Body>& b2=Body::byId(id2,scene); // not sure here
+	const shared_ptr<Body>& b1=Body::byId(id1,scene); 
+	const shared_ptr<Body>& b2=Body::byId(id2,scene); 
 
-	bool useDamping=(phys->betan!=0. || phys->betas!=0.);
+	bool useDamping=(phys->betan!=0. || phys->betas!=0. || phys->alpha!=0.);
+	bool LinDamp=true;
+	if (phys->alpha!=0.) {LinDamp=false;} // use non linear damping
 
 	// tangential and normal stiffness coefficients, recomputed from betan,betas at every step
 	Real cn=0, cs=0;
@@ -239,7 +277,7 @@ void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys
 
 	if (calcEnergy){
 		Real R=scg->radius1*scg->radius2/(scg->radius1+scg->radius2);
-		phys->radius=pow((Fn+(includeAdhesion?phys->adhesionForce:0.))*pow(R,3/2.)/phys->kno,1/3.); // not used anywhere
+		phys->radius=pow((Fn+(includeAdhesion?phys->adhesionForce:0.))*pow(R,3/2.)/phys->kno,1/3.); // attribute not used anywhere, we do not need it
 	}
 
 	/*******************************/
@@ -258,8 +296,9 @@ void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys
 	/* DAMPING COEFFICIENTS */
 	/************************/
 	
-	// Inclusion of local damping if requested 
-	if (useDamping){
+	// Inclusion of local damping if requested
+	// viscous damping is defined for both linear and non-linear elastic case 
+	if (useDamping && LinDamp){
 		Real mbar = (!b1->isDynamic() && b2->isDynamic()) ? de2->mass : ((!b2->isDynamic() && b1->isDynamic()) ? de1->mass : (de1->mass*de2->mass / (de1->mass + de2->mass))); // get equivalent mass if both bodies are dynamic, if not set it equal to the one of the dynamic body
 		//Real mbar = de1->mass*de2->mass / (de1->mass + de2->mass); // equivalent mass
 		Real Cn_crit = 2.*sqrt(mbar*phys->kn); // Critical damping coefficient (normal direction)
@@ -269,27 +308,30 @@ void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys
 		cs = Cs_crit*phys->betas; // Damping tangential coefficient
 		if(phys->kn<0 || phys->ks<0){ cerr<<"Negative stiffness kn="<<phys->kn<<" ks="<<phys->ks<<" for ##"<<b1->getId()<<"+"<<b2->getId()<<", step "<<scene->iter<<endl; }
 	}
+	else if (useDamping){ // (see Tsuji, 1992)
+		Real mbar = (!b1->isDynamic() && b2->isDynamic()) ? de2->mass : ((!b2->isDynamic() && b1->isDynamic()) ? de1->mass : (de1->mass*de2->mass / (de1->mass + de2->mass))); // get equivalent mass if both bodies are dynamic, if not set it equal to the one of the dynamic body
+		cn = phys->alpha*sqrt(mbar)*pow(uN,0.25); // normal viscous coefficient
+		cs = cn; // same value for shear viscous coefficient
+	}
 
 	/***************/
 	/* SHEAR FORCE */
 	/***************/
 	
 	Vector3r& shearElastic = phys->shearElastic; // reference for shearElastic force
-	// Define shift to handle periodicity
-	Vector3r shift2   = scene->isPeriodic ? Vector3r(                      scene->cell->Hsize *contact->cellDist.cast<Real>()) : Vector3r::Zero();
-	Vector3r shiftVel = scene->isPeriodic ? Vector3r((scene->cell->velGrad*scene->cell->Hsize)*contact->cellDist.cast<Real>()) : Vector3r::Zero();
+	// Define shifts to handle periodicity
+	const Vector3r shift2 = scene->isPeriodic ? scene->cell->intrShiftPos(contact->cellDist): Vector3r::Zero(); 
+	const Vector3r shiftVel = scene->isPeriodic ? scene->cell->intrShiftVel(contact->cellDist): Vector3r::Zero(); 
 	// 1. Rotate shear force
 	shearElastic = scg->rotate(shearElastic);
 	Vector3r prev_FsElastic = shearElastic; // save shear force at previous time step
 	// 2. Get incident velocity, get shear and normal components
-	// FIXME: Concerning the possibility to avoid granular ratcheting, it is not clear how this works in the case of HM. See the thread http://www.mail-archive.com/yade-users@lists.launchpad.net/msg01947.html
 	Vector3r incidentV = scg->getIncidentVel(de1, de2, dt, shift2, shiftVel, preventGranularRatcheting);	
 	Vector3r incidentVn = scg->normal.dot(incidentV)*scg->normal; // contact normal velocity
 	Vector3r incidentVs = incidentV - incidentVn; // contact shear velocity
 	// 3. Get shear force (incrementally)
 	shearElastic = shearElastic - phys->ks*(incidentVs*dt);
 
-
 	/**************************************/
 	/* VISCOUS DAMPING (Normal direction) */
 	/**************************************/
@@ -320,17 +362,18 @@ void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys
 	Vector3r prevUs_tot = us_total; // store previous total shear displacement (already rotated)
 	us_total -= incidentVs*dt; // add shear increment NOTE: this vector is not passed into the failure criterion, hence it holds also the plastic part of the shear displacement
 
-
 	bool noShearDamp = false; // bool to decide whether we need to account for shear damping dissipation or not
 	
 	/********************/
 	/* MOHR-COULOMB law */
 	/********************/
-	
+	phys->isSliding=false;
 	phys->shearViscous=Vector3r::Zero(); // reset so that during sliding, the previous values is not there
+	Fn = phys->normalForce.norm();
 	if (!includeAdhesion) {
 		Real maxFs = Fn*phys->tangensOfFrictionAngle;
 		if (shearElastic.squaredNorm() > maxFs*maxFs){
+			phys->isSliding=true;
 			noShearDamp = true; // no damping is added in the shear direction, hence no need to account for shear damping dissipation
 			Real ratio = maxFs/shearElastic.norm();
 			shearElastic *= ratio; phys->shearForce = shearElastic; /*store only elastic shear displacement*/ us_elastic*= ratio;
@@ -341,9 +384,10 @@ void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys
 			phys->shearForce = shearElastic - phys->shearViscous;}
 		else if (!useDamping) {phys->shearForce = shearElastic;} // update the shear force at the elastic value if no damping is present and if we passed MC
 	}
-	else { // Mohr-Coulomb formulation adpated due to the presence of adhesion (see Thornton, 1991). FIXME: is this correct?
+	else { // Mohr-Coulomb formulation adpated due to the presence of adhesion (see Thornton, 1991).
 		Real maxFs = phys->tangensOfFrictionAngle*(phys->adhesionForce+Fn); // adhesionForce already included in normalForce (above)
 		if (shearElastic.squaredNorm() > maxFs*maxFs){
+			phys->isSliding=true;
 			noShearDamp = true; // no damping is added in the shear direction, hence no need to account for shear damping dissipation
 			Real ratio = maxFs/shearElastic.norm(); shearElastic *= ratio; phys->shearForce = shearElastic; /*store only elastic shear displacement*/ us_elastic *= ratio;
 			if (calcEnergy) {frictionDissipation += (us_total-prevUs_tot).dot(shearElastic);} // calculate energy dissipation due to sliding behavior
@@ -384,61 +428,80 @@ void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys
 		scene->forces.addTorque(id1,(scg->radius1-0.5*scg->penetrationDepth)* scg->normal.cross(force));
 		scene->forces.addTorque(id2,(scg->radius2-0.5*scg->penetrationDepth)* scg->normal.cross(force));
 	}
-	phys->prevNormal = scg->normal;
-}
-
-
+	
+	/********************************************/
+	/* MOMENT CONTACT LAW */
+	/********************************************/
+	if (includeMoment){
+		// *** Bending ***//
+		// new code to compute relative particle rotation (similar to the way the shear is computed)
+		// use scg function to compute relAngVel
+		Vector3r relAngVel = scg->getRelAngVel(de1,de2,dt);
+		//Vector3r relAngVel = (b2->state->angVel-b1->state->angVel);
+		Vector3r relAngVelBend = relAngVel - scg->normal.dot(relAngVel)*scg->normal; // keep only the bending part 
+		Vector3r relRot = relAngVelBend*dt; // relative rotation due to rolling behaviour	
+		// incremental formulation for the bending moment (as for the shear part)
+		Vector3r& momentBend = phys->momentBend;
+		momentBend = scg->rotate(momentBend); // rotate moment vector (updated)
+		momentBend = momentBend-phys->kr*relRot; // add incremental rolling to the rolling vector
+		// ----------------------------------------------------------------------------------------
+		// *** Torsion ***//
+		Vector3r relAngVelTwist = scg->normal.dot(relAngVel)*scg->normal;
+		Vector3r relRotTwist = relAngVelTwist*dt; // component of relative rotation along n
+		// incremental formulation for the torsional moment
+		Vector3r& momentTwist = phys->momentTwist;
+		momentTwist = scg->rotate(momentTwist); // rotate moment vector (updated)
+		momentTwist = momentTwist-phys->ktw*relRotTwist;
 
 #if 0
-void Law2_ScGeom_MindlinPhys_Mindlin::go(shared_ptr<IGeom>& ig, shared_ptr<IPhys>& ip, Interaction* contact, Scene* ncb){
-	const Real& dt = scene->dt; // get time step
-	
-	int id1 = contact->getId1(); // get id body 1
-  	int id2 = contact->getId2(); // get id body 2
+	// code to compute the relative particle rotation
+	if (includeMoment){
+		Real rMean = (scg->radius1+scg->radius2)/2.;
+		// sliding motion
+		Vector3r duS1 = scg->radius1*(phys->prevNormal-scg->normal);
+		Vector3r duS2 = scg->radius2*(scg->normal-phys->prevNormal);
+		// rolling motion
+		Vector3r duR1 = scg->radius1*dt*b1->state->angVel.cross(scg->normal);
+		Vector3r duR2 = -scg->radius2*dt*b2->state->angVel.cross(scg->normal);
+		// relative position of the old contact point with respect to the new one
+		Vector3r relPosC1 = duS1+duR1;
+		Vector3r relPosC2 = duS2+duR2;
+		
+		Vector3r duR = (relPosC1+relPosC2)/2.; // incremental displacement vector (same radius is temporarily assumed)
 
-	State* de1 = Body::byId(id1,ncb)->state.get();
-	State* de2 = Body::byId(id2,ncb)->state.get();	
-
-	ScGeom* scg = static_cast<ScGeom*>(ig.get());
-	MindlinPhys* phys = static_cast<MindlinPhys*>(ip.get());	
-
-
-	/*** NORMAL FORCE ****/
-	Real uN = scg->penetrationDepth; // get overlapping  
-	if (uN<0) {ncb->interactions->requestErase(contact->getId1(),contact->getId2()); return;}
-	/*** Hertz-Mindlin's formulation (PFC) ***/
-	phys->kn = phys->kno*sqrt(uN); // normal stiffness
-	Real Fn = phys->kn*uN; // normal Force (scalar)
-	phys->normalForce = Fn*scg->normal; // normal Force (vector)
-
-
-	/*** SHEAR FORCE ***/
-	phys->ks = phys->kso*std::pow(Fn,1/3); // shear stiffness
-	Vector3r& trialFs = phys->shearForce;
-	Vector3r shiftVel = scene->isPeriodic ? (Vector3r)((scene->cell->velGrad*scene->cell->Hsize)*Vector3r((Real) contact->cellDist[0],(Real) contact->cellDist[1],(Real) contact->cellDist[2])) : Vector3r::Zero();
-	Vector3r shearDisp = scg->rotateAndGetShear(trialFs,phys->prevNormal,de1,de2,dt,shiftVel,preventGranularRatcheting);
-	trialFs -= phys->ks*shearDisp;
-  	
- 
-	/*** MOHR-COULOMB LAW ***/
-	Real maxFs = phys->normalForce.squaredNorm();
-	if (trialFs.squaredNorm() > maxFs)
-	{Real ratio = sqrt(maxFs)/trialFs.norm(); trialFs *= ratio;}
-	
-
-	/*** APPLY FORCES ***/
-	if (!scene->isPeriodic)
-	applyForceAtContactPoint(-phys->normalForce - trialFs, scg->contactPoint , id1, de1->se3.position, id2, de2->se3.position, ncb);
-	else { // in scg we do not wrap particles positions, hence "applyForceAtContactPoint" cannot be used
-		Vector3r force = -phys->normalForce - trialFs;
-		ncb->forces.addForce(id1,force);
-		ncb->forces.addForce(id2,-force);
-		ncb->forces.addTorque(id1,(scg->radius1-0.5*scg->penetrationDepth)* scg->normal.cross(force));
-		ncb->forces.addTorque(id2,(scg->radius2-0.5*scg->penetrationDepth)* scg->normal.cross(force));
-	}
-	phys->prevNormal = scg->normal;
-}
+		// check wheter rolling will be present, if not do nothing
+		Vector3r x=scg->normal.cross(duR);
+		Vector3r normdThetaR(Vector3r::Zero()); // initialize 
+		if(x.squaredNorm()==0) { /* no rolling */ }
+		else {
+				Vector3r normdThetaR = x/x.norm(); // moment unit vector
+				phys->dThetaR = duR.norm()/rMean*normdThetaR;} // incremental rolling
+		
+		// incremental formulation for the bending moment (as for the shear part)
+		Vector3r& momentBend = phys->momentBend;
+		momentBend = scg->rotate(momentBend); // rotate moment vector
+		momentBend = momentBend+phys->kr*phys->dThetaR; // add incremental rolling to the rolling vector FIXME: is the sign correct?
 #endif
 
+		// check plasticity condition (only bending part for the moment)
+		Real MomentMax = phys->maxBendPl*phys->normalForce.norm();
+		Real scalarMoment = phys->momentBend.norm();
+		if (MomentMax>0){
+			if(scalarMoment > MomentMax) 
+			{
+			    Real ratio = MomentMax/scalarMoment; // to fix the moment to its yielding value
+			    phys->momentBend *= ratio;
+			 }
+		}
+		// apply moments
+		Vector3r moment = phys->momentTwist+phys->momentBend;
+		scene->forces.addTorque(id1,-moment); 
+		scene->forces.addTorque(id2,moment);
+	}
+
+	// update variables
+	//phys->prevNormal = scg->normal;
+}
+
 
 
