@@ -1,12 +1,12 @@
 
-// 2009 © Václav Šmilauer <eudoxos@arcig.cz> 
+// 2009 © Václav Šmilauer <eudoxos@arcig.cz>
 
-#include<yade/pkg-dem/PeriIsoCompressor.hpp>
-#include<yade/pkg-dem/Shop.hpp>
+#include<yade/pkg/dem/PeriIsoCompressor.hpp>
+#include<yade/pkg/dem/Shop.hpp>
 #include<yade/core/Scene.hpp>
-#include<yade/pkg-common/NormShearPhys.hpp>
-#include<yade/pkg-dem/DemXDofGeom.hpp>
-#include<yade/lib-pyutil/gil.hpp>
+#include<yade/pkg/common/NormShearPhys.hpp>
+#include<yade/pkg/dem/DemXDofGeom.hpp>
+#include<yade/lib/pyutil/gil.hpp>
 
 using namespace std;
 
@@ -29,7 +29,6 @@ void PeriIsoCompressor::action(){
 			if(!b || !b->bound) continue;
 			for(int i=0; i<3; i++) maxSpan=max(maxSpan,b->bound->max[i]-b->bound->min[i]);
 		}
-		
 	}
 	if(maxDisplPerStep<0) maxDisplPerStep=1e-2*charLen; // this should be tuned somehow…
 	const long& step=scene->iter;
@@ -75,8 +74,7 @@ void PeriIsoCompressor::action(){
 	}
 	TRVAR4(cellGrow,sigma,sigmaGoal,avgStiffness);
 	assert(scene->dt>0);
-	for(int axis=0; axis<3; axis++){ scene->cell->velGrad(axis,axis)=cellGrow[axis]/(scene->dt*scene->cell->refSize[axis]); }
-	// scene->cell->refSize+=cellGrow;
+	for(int axis=0; axis<3; axis++){ scene->cell->velGrad(axis,axis)=cellGrow[axis]/(scene->dt*scene->cell->getSize()[axis]); }
 
 	// handle state transitions
 	if(allStressesOK){
@@ -95,18 +93,11 @@ void PeriIsoCompressor::action(){
 }
 
 void PeriTriaxController::strainStressStiffUpdate(){
-	// update strain first
-	//"Natural" strain, correct for large deformations, only used for comparison with goals
+	//"Natural" strain, still correct for large deformations, used for comparison with goals
 	for (int i=0;i<3;i++) strain[i]=log(scene->cell->trsf(i,i));
-	//stress tensor and stiffness
-	
+
 	//Compute volume of the deformed cell
-	// NOTE : needs refSize, could be generalized to arbitrary initial shapes using trsf*refHsize 
-	// → initial cell size is always box, and will be. The cell repeats  periodically, initial shape doesn't (shouldn't, at least) dinfluence interactions at all/
-	// → this is one more place where Hsize would make things shorter : volume=Hsize.Determinant; The full code relies on the fact that initial Hsize is a box. You didn't modify the equation btw, result is the same as in r1936, which was (with spaces...)
-	//trsf*Matrix3r ( scene->cell->refSize[0],0,0, 0,scene->cell->refSize[1],0,0,0,scene->cell->refSize[2] ) ).Determinant()
-	//remark : the volume of a parallelepiped u1,u2,u3 is always det(u1,u2,u3)
-	Real volume=scene->cell->trsf.determinant()*scene->cell->refSize[0]*scene->cell->refSize[1]*scene->cell->refSize[2];
+	Real volume=scene->cell->hSize.determinant();
 
 	//Compute sum(fi*lj) and stiffness
 	stressTensor = Matrix3r::Zero();
@@ -121,24 +112,13 @@ void PeriTriaxController::strainStressStiffUpdate(){
 		GenericSpheresContact* gsc=YADE_CAST<GenericSpheresContact*> ( I->geom.get() );
 		//Contact force
 		Vector3r f= ( reversedForces?-1.:1. ) * ( nsi->normalForce+nsi->shearForce );
-		//branch vector, FIXME : the first definition generalizes to non-spherical bodies but needs wrapped coords.
-		//    Vector3r branch=(Body::byId(I->getId1())->state->pos-Body::byId(I->getId2())->state->pos);
-		Vector3r branch= gsc->normal* ( gsc->refR1+gsc->refR2 );
-		#if 0
-			// remove this block later
-			// tensorial product f*branch (hand-write the tensor product to prevent matrix instanciation inside the loop by makeTensorProduct)
-			//stressTensor(0,0)+=f[0]*branch[0]; stressTensor(1,0)+=f[1]*branch[0]; stressTensor(2,0)+=f[2]*branch[0];
-			//stressTensor(0,1)+=f[0]*branch[1]; stressTensor(1,1)+=f[1]*branch[1]; stressTensor(2,1)+=f[2]*branch[1];
-			//stressTensor(0,2)+=f[0]*branch[2]; stressTensor(1,2)+=f[1]*branch[2]; stressTensor(2,2)+=f[2]*branch[2];
-		#endif
+		Vector3r branch=Body::byId(I->getId2(),scene)->state->pos + scene->cell->hSize*I->cellDist.cast<Real>() -Body::byId(I->getId1(),scene)->state->pos;
 		stressTensor+=f*branch.transpose();
-		if( !dynCell )
-		{
+		if( !dynCell ){
 			for ( int i=0; i<3; i++ ) sumStiff[i]+=abs ( gsc->normal[i] ) *nsi->kn+ ( 1-abs ( gsc->normal[i] ) ) *nsi->ks;
-			n++;
-		}
+			n++;}
 	}
-	// Compute stressTensor=sum(fi*lj)/Volume (Love equation)
+	// Divide by volume as in stressTensor=sum(fi*lj)/Volume (Love equation)
 	stressTensor /= volume;
 	for(int axis=0; axis<3; axis++) stress[axis]=stressTensor(axis,axis);
 	LOG_DEBUG ( "stressTensor : "<<endl
@@ -163,21 +143,22 @@ void PeriTriaxController::action()
 	//FIXME : this is wrong I think (almost sure, B.)
 	Vector3r cellArea=Vector3r(cellSize[1]*cellSize[2],cellSize[0]*cellSize[2],cellSize[0]*cellSize[1]);
 	// initial updates
-	const Vector3r& refSize=scene->cell->refSize;
 	if (maxBodySpan[0]<=0){
 		FOREACH(const shared_ptr<Body>& b,*scene->bodies){
 			if(!b || !b->bound) continue;
-			for(int i=0; i<3; i++) maxBodySpan[i]=max(maxBodySpan[i],b->bound->max[i]-b->bound->min[i]);
-		}
+			for(int i=0; i<3; i++) maxBodySpan[i]=max(maxBodySpan[i],b->bound->max[i]-b->bound->min[i]);}
 	}
 	// check current size
 	if(2.1*maxBodySpan[0]>cellSize[0] || 2.1*maxBodySpan[1]>cellSize[1] || 2.1*maxBodySpan[2]>cellSize[2]){
 		LOG_DEBUG("cellSize="<<cellSize<<", maxBodySpan="<<maxBodySpan);
-		throw runtime_error("Minimum cell size is smaller than 2.1*maxBodySpan (periodic collider requirement)");
-	}
+		throw runtime_error("Minimum cell size is smaller than 2.1*maxBodySpan (periodic collider requirement)");}
 	bool doUpdate((scene->iter%globUpdate)==0);
 	if(doUpdate || min(stiff[0],min(stiff[1],stiff[2])) <=0 || dynCell){ strainStressStiffUpdate(); }
 
+	// set mass to be sum of masses, if not set by the user
+	if(dynCell && isnan(mass)){
+		mass=0; FOREACH(const shared_ptr<Body>& b, *scene->bodies){ if(b && b->state) mass+=b->state->mass; }
+		LOG_INFO("Setting cell mass to "<<mass<<" automatically.");}
 	bool allOk=true;
 	// apply condition along each axis separately (stress or strain)
 	assert(scene->dt>0.);
@@ -193,24 +174,19 @@ void PeriTriaxController::action()
 				assert( mass>0 );//user set
 				Real dampFactor = 1 - growDamping*Mathr::Sign ( strain_rate * ( goal[axis]-stress[axis] ) );
 				strain_rate+=dampFactor*scene->dt* ( goal[axis]-stress[axis] ) /mass;
-				//if ((scene->iter%5000)==0){cerr << axis<<": stress="<<stress[axis]<<", goal="<<goal[axis]<<", velGrad="<<strain_rate<<endl;}
 				LOG_TRACE ( axis<<": stress="<<stress[axis]<<", goal="<<goal[axis]<<", velGrad="<<strain_rate );}
-			
+
 		} else {    // control strain, see "true strain" definition here http://en.wikipedia.org/wiki/Finite_strain_theory
 			///NOTE : everything could be generalized to 9 independant components by comparing F[i,i] vs. Matrix3r goal[i,i], but it would be simpler in that case to let the user set the prescribed loading rates velGrad[i,i] when [i,i] is not stress-controlled. This "else" would disappear.
 			strain_rate = (exp ( goal[axis]-strain[axis] ) -1)/scene->dt;
 			LOG_TRACE ( axis<<": strain="<<strain[axis]<<", goal="<<goal[axis]<<", cellGrow="<<strain_rate*scene->dt);
-		}		
+		}
 		// steady evolution with fluctuations; see TriaxialStressController
 		if (!dynCell) strain_rate=(1-growDamping)*strain_rate+.8*prevGrow[axis];
 		// limit maximum strain rate
 		if (abs(strain_rate)>maxStrainRate[axis]) strain_rate = Mathr::Sign(strain_rate)*maxStrainRate[axis];
 		// do not shrink below minimum cell size (periodic collider condition), although it is suboptimal WRT resulting stress
-		
-		//if ((scene->iter%5000)==0){cerr<< axis <<": velGrad="<<strain_rate<<", maxCellsize"<<-(cellSize[axis]-2.1*maxBodySpan[axis])/scene->dt<<endl;}
 		strain_rate=max(strain_rate,-(cellSize[axis]-2.1*maxBodySpan[axis])/scene->dt);
-		//if ((scene->iter%5000)==0){cerr <<"velGrad="<<strain_rate<<endl<<endl;}
-
 
 		// crude way of predicting stress, for steps when it is not computed from intrs
 		if(doUpdate) LOG_DEBUG(axis<<": cellGrow="<<strain_rate*scene->dt<<", new stress="<<stress[axis]<<", new strain="<<strain[axis]);
@@ -225,27 +201,28 @@ void PeriTriaxController::action()
 			// since strain is prescribed exactly, tolerances need just to accomodate rounding issues
 			if((goal[axis]!=0 && abs((curr-goal[axis])/goal[axis])>1e-6) || abs(curr-goal[axis])>1e-6){
 				allOk=false;
-				if(doUpdate) LOG_DEBUG("Strain not OK; "<<abs(curr-goal[axis])<<">1e-6");
-			}
+				if(doUpdate) LOG_DEBUG("Strain not OK; "<<abs(curr-goal[axis])<<">1e-6");}
 		}
 	}
 	// update stress and strain
-	if (!dynCell) for ( int axis=0; axis<3; axis++ ){		
+	if (!dynCell) for ( int axis=0; axis<3; axis++ ){
 		// take in account something like poisson's effect here…
 		//Real bogusPoisson=0.25; int ax1=(axis+1)%3,ax2=(axis+2)%3;
 		//don't modify stress if dynCell, testing only stiff[axis]>0 would not allow switching the control mode in simulations,
-		if (stiff[axis]>0) stress[axis]+=(scene->cell->velGrad(axis,axis)*scene->dt/refSize[axis])*(stiff[axis]/cellArea[axis]);
+		if (stiff[axis]>0) stress[axis]+=(scene->cell->velGrad(axis,axis)*scene->dt/cellSize[axis])*(stiff[axis]/cellArea[axis]);
 		//-bogusPoisson*(cellGrow[ax1]/refSize[ax1])*(stiff[ax1]/cellArea[ax1])-bogusPoisson*(cellGrow[ax2]/refSize[ax2])*(stiff[ax2]/cellArea[ax2]);
 	}
  	for (int k=0;k<3;k++) strainRate[k]=scene->cell->velGrad(k,k);
 	//Update energy input
-	externalWork+=(scene->cell->velGrad*stressTensor).trace()*scene->dt*scene->cell->Hsize.determinant();
+	Real dW=(scene->cell->velGrad*stressTensor).trace()*scene->dt*scene->cell->hSize.determinant();
+	externalWork+=dW;
+	if(scene->trackEnergy) scene->energy->add(-dW,"velGradWork",velGradWorkIx,/*non-incremental*/false);
 	prevGrow = strainRate;
-	
+
 	if(allOk){
 		if(doUpdate || currUnbalanced<0){
 			currUnbalanced=Shop::unbalancedForce(/*useMaxForce=*/false,scene);
-			LOG_DEBUG("Stress/strain="<<(stressMask&1?stress[0]:strain[0])<<","<<(stressMask&2?stress[1]:strain[1])<<","<<(stressMask&4?stress[2]:strain[2])<<", goal="<<goal<<", unbalanced="<<currUnbalanced );}
+			LOG_DEBUG("Stress/strain="<< (stressMask&1?stress[0]:strain[0]) <<"," <<(stressMask&2?stress[1]:strain[1])<<"," <<(stressMask&4?stress[2]:strain[2]) <<", goal="<<goal<<", unbalanced="<<currUnbalanced );}
 		if(currUnbalanced<maxUnbalanced){
 			// LOG_INFO("Goal reached, packing stable.");
 			if (!doneHook.empty()){
@@ -256,9 +233,6 @@ void PeriTriaxController::action()
 	}
 }
 
-
-
-
 CREATE_LOGGER(Peri3dController);
 void Peri3dController::action(){
 	if(!scene->isPeriodic){ LOG_FATAL("Being used on non-periodic simulation!"); throw; }
@@ -300,7 +274,7 @@ void Peri3dController::action(){
 		// PATH_OP_OP(0,j,k) = path[0]->operator[](j).operator(k) is k-th element of j-th Vector2r of xxPath
 		#define PATH_OP_OP(pi,op1i,op2i) paths[pi]->operator[](op1i).operator()(op2i)
 
-		for (int i=0; i<6; i++) { 
+		for (int i=0; i<6; i++) {
 			for (int j=1; j<pathSizes[i]; j++) {
 				// check if the user defined time axis is monothonically increasing
 				{ if ( PATH_OP_OP(i,j-1,0) >= PATH_OP_OP(i,j,0) ) {
@@ -319,12 +293,12 @@ void Peri3dController::action(){
 
 		// set weather the simulation is "stress based" (all stress components prescribed or all prescribed strains equal zero)
 		if (lenPe == 0) { stressBasedSimulation = true; }
-		else { 
+		else {
 			stressBasedSimulation = true;
 			for (int i=0; i<lenPe; i++) { stressBasedSimulation = stressBasedSimulation && PATH_OP_OP(pe(i),1,1)<1e9; }
 		}
 	}
-	
+
 	// increase the pathCounter by one if we cross to the next part of path
 	for (int i=0; i<6; i++) {
 		if (progress >= PATH_OP_OP(i,pathsCounter[i],0)) { pathsCounter[i]++; }
@@ -410,7 +384,7 @@ void Peri3dController::action(){
 			}
 		}
 	}
-	
+
 	// correction coefficient ix strainRate.maxabs() > maxStrainRate
 	Real srCorr = (strainRate.cwise().abs().maxCoeff() > maxStrainRate)? (maxStrainRate/strainRate.cwise().abs().maxCoeff()):1.;
 	strainRate *= srCorr;
@@ -419,12 +393,12 @@ void Peri3dController::action(){
 	const Matrix3r& trsf=scene->cell->trsf;
 	// compute rotational and nonrotational (strain in local coordinates) part of trsf
 	Matrix_computeUnitaryPositive(trsf,&rot,&nonrot);
-	 
+
 	// prescribed velocity gradient (strain tensor rate) in global coordinates
-	epsilonRate = voigt_toSymmTensor(strainRate,/*strain=*/true); 
+	epsilonRate = voigt_toSymmTensor(strainRate,/*strain=*/true);
 	/* transformation of prescribed strain rate (computed by predictor) into local cell coordinates,
 	   multiplying by time to obtain strain increment and adding it to nonrot (current strain in local coordinates)*/
-	nonrot += rot.transpose()*(epsilonRate*dt)*rot; 
+	nonrot += rot.transpose()*(epsilonRate*dt)*rot;
 	Matrix3r& velGrad=scene->cell->velGrad;
 	// compute new trsf as rot*nonrot, substract actual trsf (= trsf increment), divide by dt (=trsf rate = velGrad
 	//trsf = rot*nonrot;
@@ -506,7 +480,7 @@ void Peri3dController::action(){
 	typedef Eigen::Matrix<Real,Eigen::Dynamic,Eigen::Dynamic> MatrixXr;
 	typedef Eigen::Matrix<Real,Eigen::Dynamic,1> VectorXr;
 	const Real& dt=scene->dt;
-	/ * 
+	/ *
 	sigma = K * eps
 
 	decompose the stiffness matrix depending on what is prescribed
