@@ -1,11 +1,11 @@
 // © 2009 Václav Šmilauer <eudoxos@arcig.cz>
 
-#include<yade/pkg-dem/SpherePack.hpp>
+#include<yade/pkg/dem/SpherePack.hpp>
 
 #include<yade/core/Omega.hpp>
 #include<yade/core/Scene.hpp>
-#include<yade/pkg-common/Sphere.hpp>
-#include<yade/pkg-dem/Shop.hpp>
+#include<yade/pkg/common/Sphere.hpp>
+#include<yade/pkg/dem/Shop.hpp>
 
 #include <boost/random/linear_congruential.hpp>
 #include <boost/random/uniform_real.hpp>
@@ -54,7 +54,7 @@ void SpherePack::fromFile(string file) {
 	typedef pair<Vector3r,Real> pairVector3rReal;
 	vector<pairVector3rReal> ss;
 	Vector3r mn,mx;
-	ss=Shop::loadSpheresFromFile(file,mn,mx);
+	ss=Shop::loadSpheresFromFile(file,mn,mx,&cellSize);
 	pack.clear();
 	FOREACH(const pairVector3rReal& s, ss) pack.push_back(Sph(s.first,s.second));
 }
@@ -62,6 +62,7 @@ void SpherePack::fromFile(string file) {
 void SpherePack::toFile(const string fname) const {
 	ofstream f(fname.c_str());
 	if(!f.good()) throw runtime_error("Unable to open file `"+fname+"'");
+	if(cellSize!=Vector3r::Zero()){ f<<"##PERIODIC:: "<<cellSize[0]<<" "<<cellSize[1]<<" "<<cellSize[2]<<endl; }
 	FOREACH(const Sph& s, pack){
 		if(s.clumpId>=0) throw std::invalid_argument("SpherePack with clumps cannot be (currently) exported to a text file.");
 		f<<s.c[0]<<" "<<s.c[1]<<" "<<s.c[2]<<" "<<s.r<<endl;
@@ -81,21 +82,27 @@ void SpherePack::fromSimulation() {
 	if(scene->isPeriodic) { cellSize=scene->cell->getSize(); }
 }
 
-long SpherePack::makeCloud(Vector3r mn, Vector3r mx, Real rMean, Real rRelFuzz, int num, bool periodic, Real porosity, const vector<Real>& psdSizes, const vector<Real>& psdCumm, bool distributeMass){
-	static boost::minstd_rand randGen(TimingInfo::getNow(/* get the number even if timing is disabled globally */ true));
+long SpherePack::makeCloud(Vector3r mn, Vector3r mx, Real rMean, Real rRelFuzz, int num, bool periodic, Real porosity, const vector<Real>& psdSizes, const vector<Real>& psdCumm, bool distributeMass, int seed, Matrix3r hSize){
+	static boost::minstd_rand randGen(seed!=0?seed:(int)TimingInfo::getNow(/* get the number even if timing is disabled globally */ true));
 	static boost::variate_generator<boost::minstd_rand&, boost::uniform_real<Real> > rnd(randGen, boost::uniform_real<Real>(0,1));
-	vector<Real> psdRadii; // hold psdSizes as volume if distributeMass is in use, as plain radii (rather than diameters) if not
-	vector<Real> psdCumm2; // psdCumm but dimensionally transformed to match mass distribution
+	vector<Real> psdRadii; // holds plain radii (rather than diameters), scaled down in some situations to get the target number
+	vector<Real> psdCumm2; // psdCumm but dimensionally transformed to match mass distribution	
+	Vector3r size;
+	bool hSizeFound =(hSize!=Matrix3r::Zero());//is hSize passed to the function?
+	if (!hSizeFound) {size=mx-mn; hSize=size.asDiagonal();}
+	if (hSizeFound && !periodic) LOG_WARN("hSize can be defined only for periodic cells.");
+	Matrix3r invHsize =hSize.inverse();
+	Real volume=hSize.determinant();
+	if (!volume) throw invalid_argument("The box defined as null volume. Define at least maxCorner of the box, or hSize if periodic.");
 	int mode=-1; bool err=false;
 	// determine the way we generate radii
+	if(porosity<=0) {LOG_WARN("porosity must be >0, changing it for you. It will be ineffective if rMean>0."); porosity=0.5;}
+	//If rMean is not defined, then in will be defined in RDIST_NUM
 	if(rMean>0) mode=RDIST_RMEAN;
-	if(porosity>0){
-		err=(mode>=0); mode=RDIST_POROSITY;
-		if(num<0) throw invalid_argument("SpherePack.makeCloud: when specifying porosity, num must be given as well.");
-		Vector3r dimensions=mx-mn; Real volume=dimensions.x()*dimensions.y()*dimensions.z();
+	else if(num>0 && psdSizes.size()==0) {
+		mode=RDIST_NUM;
 		// the term (1+rRelFuzz²) comes from the mean volume for uniform distribution : Vmean = 4/3*pi*Rmean*(1+rRelFuzz²) 
-		rMean=pow(volume*(1-porosity)/(Mathr::PI*(4/3.)*(1+rRelFuzz*rRelFuzz)*num),1/3.);
-	}
+		rMean=pow(volume*(1-porosity)/(Mathr::PI*(4/3.)*(1+rRelFuzz*rRelFuzz)*num),1/3.);}
 	if(psdSizes.size()>0){
 		err=(mode>=0); mode=RDIST_PSD;
 		if(psdSizes.size()!=psdCumm.size()) throw invalid_argument(("SpherePack.makeCloud: psdSizes and psdCumm must have same dimensions ("+lexical_cast<string>(psdSizes.size())+"!="+lexical_cast<string>(psdCumm.size())).c_str());
@@ -104,59 +111,89 @@ long SpherePack::makeCloud(Vector3r mn, Vector3r mx, Real rMean, Real rRelFuzz,
 		psdRadii.reserve(psdSizes.size());
 		for(size_t i=0; i<psdSizes.size(); i++) {
 			psdRadii.push_back(/* radius, not diameter */ .5*psdSizes[i]);
-			if(distributeMass) psdCumm2.push_back(1-pow(1-psdCumm[i],psdScaleExponent)); // 5/2. is an approximate (fractal) dimension, empirically determined
+			if(distributeMass) {
+				//psdCumm2 is first obtained by integrating the number of particles over the volumic PSD (dN/dSize = totV*(dPassing/dSize)*1/(4/3πr³)). I suspect similar reasoning behind pow3Interp, since the expressions are a bit similar. The total cumulated number will be the number of spheres in volume*(1-porosity), it is used to decide if the PSD will be scaled down. psdCumm2 is normalized below in order to fit in [0,1]. (B.C.)
+				if (i==0) psdCumm2.push_back(0);
+				else psdCumm2.push_back(psdCumm2[i-1] + 3.0*volume*(1-porosity)/Mathr::PI*(psdCumm[i]-psdCumm[i-1])/(psdSizes[i]-psdSizes[i-1])*(pow(psdSizes[i-1],-2)-pow(psdSizes[i],-2)));
+			}
 			LOG_DEBUG(i<<". "<<psdRadii[i]<<", cdf="<<psdCumm[i]<<", cdf2="<<(distributeMass?lexical_cast<string>(psdCumm2[i]):string("--")));
 			// check monotonicity
 			if(i>0 && (psdSizes[i-1]>psdSizes[i] || psdCumm[i-1]>psdCumm[i])) throw invalid_argument("SpherePack:makeCloud: psdSizes and psdCumm must be both non-decreasing.");
 		}
+		if(distributeMass) {
+			if (num>0) {
+				//if target number will not fit in (1-poro)*volume, scale down particles size
+				if (psdCumm2[psdSizes.size()-1]<num){
+					appliedPsdScaling=pow(psdCumm2[psdSizes.size()-1]/num,1./3.);
+					for(size_t i=0; i<psdSizes.size(); i++) psdRadii[i]*=appliedPsdScaling;}
+			}
+			//Normalize psdCumm2 so it's between 0 and 1
+			for(size_t i=1; i<psdSizes.size(); i++) psdCumm2[i]/=psdCumm2[psdSizes.size()-1];
+		}
 	}
-	if(err || mode<0) throw invalid_argument("SpherePack.makeCloud: exactly one of rMean, porosity, psdSizes & psdCumm arguments must be specified.");
+	if(err || mode<0) throw invalid_argument("SpherePack.makeCloud: at least one of rMean, porosity, psdSizes & psdCumm arguments must be specified. rMean can't be combined with psdSizes.");
 	// adjust uniform distribution parameters with distributeMass; rMean has the meaning (dimensionally) of _volume_
 	const int maxTry=1000;
-	Vector3r size=mx-mn;
 	if(periodic)(cellSize=size);
 	Real r=0;
 	for(int i=0; (i<num) || (num<0); i++) {
-		// determine radius of the next sphere we will attempt to place in space
+		Real norm, rand;
+		//Determine radius of the next sphere we will attempt to place in space. If (num>0), generate radii the deterministic way, in decreasing order, else radii are stochastic since we don't know what the final number will be
+		if (num>0) rand = ((Real)num-(Real)i+0.5)/((Real)num+1.);
+		else rand = rnd();
 		int t;
 		switch(mode){
 			case RDIST_RMEAN:
-			case RDIST_POROSITY: 
-				if(distributeMass) r=pow3Interp(rnd(),rMean*(1-rRelFuzz),rMean*(1+rRelFuzz));
-				else r=rMean*(2*(rnd()-.5)*rRelFuzz+1); // uniform distribution in rMean*(1±rRelFuzz)
+				//FIXME : r is never defined, it will be zero at first iteration, but it will have values in the next ones. Some magic?
+			case RDIST_NUM:
+				if(distributeMass) r=pow3Interp(rand,rMean*(1-rRelFuzz),rMean*(1+rRelFuzz));
+				else r=rMean*(2*(rand-.5)*rRelFuzz+1); // uniform distribution in rMean*(1±rRelFuzz)
 				break;
 			case RDIST_PSD:
 				if(distributeMass){
-					Real norm;
-					int piece=psdGetPiece(rnd(),psdCumm2,norm);
+					int piece=psdGetPiece(rand,psdCumm2,norm);
 					r=pow3Interp(norm,psdRadii[piece],psdRadii[piece+1]);
 				} else {
-					Real norm; int piece=psdGetPiece(rnd(),psdCumm,norm);
-					r=psdRadii[piece]+norm*(psdRadii[piece+1]-psdRadii[piece]);
-				}
+					int piece=psdGetPiece(rand,psdCumm,norm);
+					r=psdRadii[piece]+norm*(psdRadii[piece+1]-psdRadii[piece]);}
 		}
 		// try to put the sphere into a free spot
 		for(t=0; t<maxTry; ++t){
 			Vector3r c;
 			if(!periodic) { for(int axis=0; axis<3; axis++) c[axis]=mn[axis]+r+(size[axis]-2*r)*rnd(); }
-			else { for(int axis=0; axis<3; axis++) c[axis]=mn[axis]+size[axis]*rnd(); }
+			else { 	for(int axis=0; axis<3; axis++) c[axis]=rnd();//coordinates in [0,1]
+				c=mn+hSize*c;}//coordinates in reference frame (inside the base cell)
 			size_t packSize=pack.size(); bool overlap=false;
-			if(!periodic){
-				for(size_t j=0; j<packSize; j++){ if(pow(pack[j].r+r,2) >= (pack[j].c-c).squaredNorm()) { overlap=true; break; } }
-			} else {
+			if(!periodic) for(size_t j=0;j<packSize;j++) {if(pow(pack[j].r+r,2)>=(pack[j].c-c).squaredNorm()) {overlap=true; break;}}
+			else {
 				for(size_t j=0; j<packSize; j++){
-					Vector3r dr;
-					for(int axis=0; axis<3; axis++) dr[axis]=min(cellWrapRel(c[axis],pack[j].c[axis],pack[j].c[axis]+size[axis]),cellWrapRel(pack[j].c[axis],c[axis],c[axis]+size[axis]));
+					Vector3r dr=Vector3r::Zero();
+					if (!hSizeFound) {//The box is axis-aligned, use the wrap methods
+						for(int axis=0; axis<3; axis++) dr[axis]=min(cellWrapRel(c[axis],pack[j].c[axis],pack[j].c[axis]+size[axis]),cellWrapRel(pack[j].c[axis],c[axis],c[axis]+size[axis]));
+					} else {//not aligned, find closest neighbor in a cube of size 1, then transform distance to cartesian coordinates
+						Vector3r c1c2=invHsize*(pack[j].c-c);
+						for(int axis=0; axis<3; axis++){
+							if (abs(c1c2[axis])<abs(c1c2[axis] - Mathr::Sign(c1c2[axis]))) dr[axis]=c1c2[axis];
+							else dr[axis] = c1c2[axis] - Mathr::Sign(c1c2[axis]);}
+						dr=hSize*dr;//now in cartesian coordinates
+					}
 					if(pow(pack[j].r+r,2)>= dr.squaredNorm()){ overlap=true; break; }
 				}
 			}
 			if(!overlap) { pack.push_back(Sph(c,r)); break; }
 		}
 		if (t==maxTry) {
-			if(num>0) LOG_WARN("Exceeded "<<maxTry<<" tries to insert non-overlapping sphere to packing. Only "<<i<<" spheres was added, although you requested "<<num);
-			return i;
-		}
+			if(num>0) {
+				if (mode!=RDIST_RMEAN) {
+					Real nextPoro = porosity+(1-porosity)/10.;
+					LOG_WARN("Exceeded "<<maxTry<<" tries to insert non-overlapping sphere to packing. Only "<<i<<" spheres was added, although you requested "<<num<<". Trying again with porosity "<<nextPoro<<". The size distribution is being scaled down");
+					pack.clear();
+					return makeCloud(mn, mx, -1., rRelFuzz, num, periodic, nextPoro, psdSizes, psdCumm, distributeMass,seed,hSizeFound?hSize:Matrix3r::Zero());}
+				else LOG_WARN("Exceeded "<<maxTry<<" tries to insert non-overlapping sphere to packing. Only "<<i<<" spheres was added, although you requested "<<num<<".");
+			}
+			return i;}
 	}
+	if (appliedPsdScaling<1) LOG_WARN("The size distribution has been scaled down by a factor pack.appliedPsdScaling="<<appliedPsdScaling);
 	return pack.size();
 }
 
@@ -216,8 +253,54 @@ py::tuple SpherePack::psd(int bins, bool mass) const {
 	return py::make_tuple(edges,cumm);
 }
 
-// New code to include the psd giving few points of it
-long SpherePack::particleSD(Vector3r mn, Vector3r mx, Real rMean, bool periodic, string name, int numSph, const vector<Real>& radii, const vector<Real>& passing, bool passingIsNotPercentageButCount){
+/* possible enhacement: proportions parameter, so that the domain is not cube, but box with sides having given proportions */
+long SpherePack::particleSD2(const vector<Real>& radii, const vector<Real>& passing, int numSph, bool periodic, Real cloudPorosity, int seed){
+	typedef Eigen::Matrix<Real,Eigen::Dynamic,Eigen::Dynamic> MatrixXr;
+	typedef Eigen::Matrix<Real,Eigen::Dynamic,1> VectorXr;
+	
+	int dim=radii.size()+1;
+	if(passing.size()!=radii.size()) throw std::invalid_argument("SpherePack.particleSD2: radii and passing must have the same length.");
+	MatrixXr M=MatrixXr::Zero(dim,dim);
+	VectorXr rhs=VectorXr::Zero(dim);
+	/*
+	
+	We know percentages for each fraction (Δpi) and their radii (ri), and want to find
+	the number of sphere for each fraction Ni and total solid volume Vs. For each fraction,
+	we know that the volume is equal to Ni*(4/3*πri³), which must be equal to Vs*Δpi (Δpi is
+	relative solid volume of the i-th fraction).
+
+	The last equation says that total number of particles (sum of fractions) is equal to N,
+	which is the total number of particles requested by the user.
+
+	   N1     N2     N3    Vs       rhs
+
+	4/3πr₁³   0      0     -Δp₁   | 0
+	  0     4/3πr₂³  0     -Δp₂   | 0
+	  0       0    4/3πr₃³ -Δp₃   | 0
+     1       1      1      0     | N
+
+	*/
+	for(int i=0; i<dim-1; i++){
+		M(i,i)=(4/3.)*Mathr::PI*pow(radii[i],3);
+		M(i,dim-1)=-(passing[i]-(i>0?passing[i-1]:0))/100.;
+		M(dim-1,i)=1;
+	}
+	rhs[dim-1]=numSph;
+	// NumsVs=M^-1*rhs: number of spheres and volume of solids
+	VectorXr NumsVs(dim); NumsVs=M.inverse()*rhs;
+	Real Vs=NumsVs[dim-1]; // total volume of solids
+	Real Vtot=Vs/(1-cloudPorosity); // total volume of cell containing the packing
+	Vector3r cellSize=pow(Vtot,1/3.)*Vector3r().Ones(); // for now, assume always cubic sample
+	Real rMean=pow(Vs/(numSph*(4/3.)*Mathr::PI),1/3.); // make rMean such that particleSD will compute the right Vs (called a bit confusingly Vtot anyway) inversely
+	// cerr<<"Vs="<<Vs<<", Vtot="<<Vtot<<", rMean="<<rMean<<endl;
+	// cerr<<"cellSize="<<cellSize<<", rMean="<<rMean<<", numSph="<<numSph<<endl;
+	return particleSD(Vector3r::Zero(),cellSize,rMean,periodic,"",numSph,radii,passing,false);
+};
+
+// TODO: header, python wrapper, default params
+
+// Discrete particle size distribution
+long SpherePack::particleSD(Vector3r mn, Vector3r mx, Real rMean, bool periodic, string name, int numSph, const vector<Real>& radii, const vector<Real>& passing, bool passingIsNotPercentageButCount, int seed){
 	vector<Real> numbers;
 	if(!passingIsNotPercentageButCount){
 		Real Vtot=numSph*4./3.*Mathr::PI*pow(rMean,3.); // total volume of the packing (computed with rMean)
@@ -228,18 +311,13 @@ long SpherePack::particleSD(Vector3r mn, Vector3r mx, Real rMean, bool periodic,
 			Real volS=4./3.*Mathr::PI*pow(radii[i],3.);
 			if (i==0) {numbers.push_back(passing[i]/100.*Vtot/volS);}
 			else {numbers.push_back((passing[i]-passing[i-1])/100.*Vtot/volS);} // 
-			
-			cout<<"numbers["<<i<<"] = "<<numbers[i]<<endl;
-			cout<<"radii["<<i<<"] = "<<radii[i]<<endl;
-			cout<<"vol tot = "<<Vtot<<endl;
-			cout<<"v_sphere = "<<volS<<endl;
-			cout<<"passing["<<i<<"] = "<<passing[i]<<endl;
+			cout<<"fraction #"<<i<<" ("<<passing[i]<<"%, r="<<radii[i]<<"): "<<numbers[i]<<" spheres, fraction/cloud volumes "<<volS<<"/"<<Vtot<<endl;
 		}
 	} else {
 		FOREACH(Real p, passing) numbers.push_back(p);
 	}
 
-	static boost::minstd_rand randGen(TimingInfo::getNow(true));
+	static boost::minstd_rand randGen(seed!=0?seed:(int)TimingInfo::getNow(true));
 	static boost::variate_generator<boost::minstd_rand&, boost::uniform_real<> > rnd(randGen, boost::uniform_real<>(0,1));
 
 	const int maxTry=1000;
@@ -266,7 +344,60 @@ long SpherePack::particleSD(Vector3r mn, Vector3r mx, Real rMean, bool periodic,
 				if(!overlap) { pack.push_back(Sph(c,r)); break; }
 			}
 			if (t==maxTry) {
-				if(numbers[ii]>0) LOG_WARN("Exceeded "<<maxTry<<" tries to insert non-overlapping sphere to packing. Only "<<i<<" spheres was added, although you requested "<<numbers[ii]<<" with radius"<<radii[ii]);
+				if(numbers[ii]>0) LOG_WARN("Exceeded "<<maxTry<<" tries to insert non-overlapping sphere to packing. Only "<<i<<" spheres was added, although you requested "<<numbers[ii]<<" with radius "<<radii[ii]);
+				return i;
+			}
+		}
+	}
+	return pack.size();
+}
+
+// 2d function
+long SpherePack::particleSD_2d(Vector2r mn, Vector2r mx, Real rMean, bool periodic, string name, int numSph, const vector<Real>& radii, const vector<Real>& passing, bool passingIsNotPercentageButCount, int seed){
+	vector<Real> numbers;
+	if(!passingIsNotPercentageButCount){
+		Real Vtot=numSph*4./3.*Mathr::PI*pow(rMean,3.); // total volume of the packing (computed with rMean)
+		
+		// calculate number of spheres necessary per each radius to match the wanted psd
+		// passing has to contain increasing values
+		for (size_t i=0; i<radii.size(); i++){
+			Real volS=4./3.*Mathr::PI*pow(radii[i],3.);
+			if (i==0) {numbers.push_back(passing[i]/100.*Vtot/volS);}
+			else {numbers.push_back((passing[i]-passing[i-1])/100.*Vtot/volS);} // 
+			cout<<"fraction #"<<i<<" ("<<passing[i]<<"%, r="<<radii[i]<<"): "<<numbers[i]<<" spheres, fraction/cloud volumes "<<volS<<"/"<<Vtot<<endl;
+		}
+	} else {
+		FOREACH(Real p, passing) numbers.push_back(p);
+	}
+
+	static boost::minstd_rand randGen(seed!=0?seed:(int)TimingInfo::getNow(true));
+	static boost::variate_generator<boost::minstd_rand&, boost::uniform_real<> > rnd(randGen, boost::uniform_real<>(0,1));
+
+	const int maxTry=1000;
+	Vector2r size=mx-mn; 
+	//if(periodic)(cellSize=size); in this case, it must be defined in py script as cell needs the third dimension
+	for (int ii=(int)radii.size()-1; ii>=0; ii--){
+		Real r=radii[ii]; // select radius
+		for(int i=0; i<numbers[ii]; i++) { // place as many spheres as required by the psd for the selected radius into the free spot
+			int t;
+			for(t=0; t<maxTry; ++t){
+				Vector3r c;
+				if(!periodic) { for(int axis=0; axis<2; axis++) c[axis]=mn[axis]+r+(size[axis]-2*r)*rnd(); }
+				else { for(int axis=0; axis<2; axis++) c[axis]=mn[axis]+size[axis]*rnd(); }
+				size_t packSize=pack.size(); bool overlap=false;
+				if(!periodic){
+					for(size_t j=0; j<packSize; j++){ if(pow(pack[j].r+r,2) >= (pack[j].c-c).squaredNorm()) { overlap=true; break; } }
+				} else {
+					for(size_t j=0; j<packSize; j++){
+						Vector3r dr;
+						for(int axis=0; axis<2; axis++) dr[axis]=min(cellWrapRel(c[axis],pack[j].c[axis],pack[j].c[axis]+size[axis]),cellWrapRel(pack[j].c[axis],c[axis],c[axis]+size[axis]));
+						if(pow(pack[j].r+r,2)>= dr.squaredNorm()){ overlap=true; break; }
+					}
+				}
+				if(!overlap) { pack.push_back(Sph(c,r)); break; }
+			}
+			if (t==maxTry) {
+				if(numbers[ii]>0) LOG_WARN("Exceeded "<<maxTry<<" tries to insert non-overlapping sphere to packing. Only "<<i<<" spheres was added, although you requested "<<numbers[ii]<<" with radius "<<radii[ii]);
 				return i;
 			}
 		}
@@ -282,9 +413,9 @@ long SpherePack::makeClumpCloud(const Vector3r& mn, const Vector3r& mx, const ve
 		SpherePack c2(*c); 
 		c2.translate(c2.midPt()); //recenter
 		clumps.push_back(c2);
-		Real rSq=0;
-		FOREACH(const Sph& s, c2.pack) rSq=max(rSq,s.c.squaredNorm()+pow(s.r,2));
-		boundRad.push_back(sqrt(rSq));
+		Real r=0;
+		FOREACH(const Sph& s, c2.pack) r=max(r,s.c.norm()+s.r);
+		boundRad.push_back(r);
 		Vector3r cMn,cMx; c2.aabb(cMn,cMx); // centered at zero now, this gives margin
 		//margins.push_back(periodic?cMx:Vector3r::Zero()); 
 		//boxMargins=boxMargins.cwise().max(cMx);
