@@ -1,3 +1,5 @@
+#ifdef YADE_VTK
+
 #include"VTKRecorder.hpp"
 
 // avoid warnings in VTK code (does not work?!)
@@ -22,15 +24,15 @@
 #pragma GCC diagnostic warning "-Wdeprecated"
 
 #include<yade/core/Scene.hpp>
-#include<yade/pkg-common/Sphere.hpp>
-#include<yade/pkg-common/Facet.hpp>
-#include<yade/pkg-dem/ConcretePM.hpp>
-#include<yade/pkg-dem/RockPM.hpp>
-#include<yade/pkg-dem/Shop.hpp>
+#include<yade/pkg/common/Sphere.hpp>
+#include<yade/pkg/common/Facet.hpp>
+#include<yade/pkg/dem/ConcretePM.hpp>
+#include<yade/pkg/dem/RockPM.hpp>
+#include<yade/pkg/dem/WirePM.hpp>
+#include<yade/pkg/dem/Shop.hpp>
 
 
 YADE_PLUGIN((VTKRecorder));
-YADE_REQUIRE_FEATURE(VTK)
 CREATE_LOGGER(VTKRecorder);
 
 void VTKRecorder::action(){
@@ -54,17 +56,22 @@ void VTKRecorder::action(){
 		else if((rec=="colors") || (rec=="color"))recActive[REC_COLORS]=true;
 		else if(rec=="cpm") recActive[REC_CPM]=true;
 		else if(rec=="rpm") recActive[REC_RPM]=true;
+		else if(rec=="wpm") recActive[REC_WPM]=true;
 		else if(rec=="intr") recActive[REC_INTR]=true;
 		else if((rec=="ids") || (rec=="id")) recActive[REC_ID]=true;
 		else if(rec=="mask") recActive[REC_MASK]=true;
 		else if((rec=="clumpids") || (rec=="clumpId")) recActive[REC_CLUMPID]=true;
 		else if(rec=="materialId") recActive[REC_MATERIALID]=true;
 		else if(rec=="stress") recActive[REC_STRESS]=true;
-		else LOG_ERROR("Unknown recorder named `"<<rec<<"' (supported are: all, spheres, velocity, facets, color, stress, cpm, rpm, intr, id, clumpId, materialId). Ignored.");
+		else LOG_ERROR("Unknown recorder named `"<<rec<<"' (supported are: all, spheres, velocity, facets, color, stress, cpm, rpm, wpm, intr, id, clumpId, materialId). Ignored.");
 	}
 	// cpm needs interactions
 	if(recActive[REC_CPM]) recActive[REC_INTR]=true;
 
+	// wpm needs interactions
+	if(recActive[REC_WPM]) recActive[REC_INTR]=true;
+
+	
 	// spheres
 	vtkSmartPointer<vtkPoints> spheresPos = vtkSmartPointer<vtkPoints>::New();
 	vtkSmartPointer<vtkCellArray> spheresCells = vtkSmartPointer<vtkCellArray>::New();
@@ -159,15 +166,9 @@ void VTKRecorder::action(){
 	vtkSmartPointer<vtkFloatArray> cpmDamage = vtkSmartPointer<vtkFloatArray>::New();
 	cpmDamage->SetNumberOfComponents(1);
 	cpmDamage->SetName("cpmDamage");
-	vtkSmartPointer<vtkFloatArray> cpmSigma = vtkSmartPointer<vtkFloatArray>::New();
-	cpmSigma->SetNumberOfComponents(3);
-	cpmSigma->SetName("cpmSigma");
-	vtkSmartPointer<vtkFloatArray> cpmSigmaM = vtkSmartPointer<vtkFloatArray>::New();
-	cpmSigmaM->SetNumberOfComponents(1);
-	cpmSigmaM->SetName("cpmSigmaM");
-	vtkSmartPointer<vtkFloatArray> cpmTau = vtkSmartPointer<vtkFloatArray>::New();
-	cpmTau->SetNumberOfComponents(3);
-	cpmTau->SetName("cpmTau");
+	vtkSmartPointer<vtkFloatArray> cpmStress = vtkSmartPointer<vtkFloatArray>::New();
+	cpmStress->SetNumberOfComponents(9);
+	cpmStress->SetName("cpmStress");
 	
 	// extras for RPM
 	vtkSmartPointer<vtkFloatArray> rpmSpecNum = vtkSmartPointer<vtkFloatArray>::New();
@@ -179,6 +180,11 @@ void VTKRecorder::action(){
 	vtkSmartPointer<vtkFloatArray> rpmSpecDiam = vtkSmartPointer<vtkFloatArray>::New();
 	rpmSpecDiam->SetNumberOfComponents(1);
 	rpmSpecDiam->SetName("rpmSpecDiam");
+	
+	// extras for WireMatPM
+	vtkSmartPointer<vtkFloatArray> wpmLimitNormalFactor = vtkSmartPointer<vtkFloatArray>::New();
+	wpmLimitNormalFactor->SetNumberOfComponents(1);
+	wpmLimitNormalFactor->SetName("wpmLimitNormalFactor");
 
 	if(recActive[REC_INTR]){
 		// holds information about cell distance between spatial and displayed position of each particle
@@ -229,8 +235,8 @@ void VTKRecorder::action(){
 				// create two line objects; each of them has one endpoint inside the cell and the other one sticks outside
 				// A,B are the "fake" bodies outside the cell for id1 and id2 respectively, p1,p2 are the displayed points
 				// distance in cell units for shifting A away from p1; negated value is shift of B away from p2
-				Vector3r ptA(p01+scene->cell->Hsize*(wrapCellDist[I->getId2()]-I->cellDist).cast<Real>());
-				Vector3r ptB(p02+scene->cell->Hsize*(wrapCellDist[I->getId1()]-I->cellDist).cast<Real>());
+				Vector3r ptA(p01+scene->cell->hSize*(wrapCellDist[I->getId2()]-I->cellDist).cast<Real>());
+				Vector3r ptB(p02+scene->cell->hSize*(wrapCellDist[I->getId1()]-I->cellDist).cast<Real>());
 				vtkIdType idPtA=intrBodyPos->InsertNextPoint(ptA[0],ptA[1],ptA[2]), idPtB=intrBodyPos->InsertNextPoint(ptB[0],ptB[1],ptB[2]);
 				vtkSmartPointer<vtkLine> line1B(vtkSmartPointer<vtkLine>::New()); line1B->GetPointIds()->SetId(0,I->getId1()); line1B->GetPointIds()->SetId(1,idPtB);
 				vtkSmartPointer<vtkLine> lineA2(vtkSmartPointer<vtkLine>::New()); lineA2->GetPointIds()->SetId(0,idPtA); line1B->GetPointIds()->SetId(1,I->getId2());
@@ -245,6 +251,10 @@ void VTKRecorder::action(){
 			for(int i=0; i<numAddValues; i++){
 				intrForceN->InsertNextValue(fn);
 				intrAbsForceT->InsertNextTupleValue(fs);
+				if(recActive[REC_WPM]) {
+					const WirePhys* wirephys = YADE_CAST<WirePhys*>(I->phys.get());
+					wpmLimitNormalFactor->InsertNextValue(wirephys->limitNormalFactor);
+				}
 			}
 		}
 	}
@@ -259,7 +269,7 @@ void VTKRecorder::action(){
 		if (recActive[REC_SPHERES]){
 			const Sphere* sphere = dynamic_cast<Sphere*>(b->shape.get()); 
 			if (sphere){
-				if(skipNondynamic && !b->isDynamic()) continue;
+				if(skipNondynamic && b->state->blockedDOFs==State::DOF_ALL) continue;
 				vtkIdType pid[1];
 				Vector3r pos(scene->isPeriodic ? scene->cell->wrapShearedPt(b->state->pos) : b->state->pos);
 				pid[0] = spheresPos->InsertNextPoint(pos[0], pos[1], pos[2]);
@@ -296,13 +306,10 @@ void VTKRecorder::action(){
 				
 				if (recActive[REC_CPM]){
 					cpmDamage->InsertNextValue(YADE_PTR_CAST<CpmState>(b->state)->normDmg);
-					const Vector3r& ss=YADE_PTR_CAST<CpmState>(b->state)->sigma;
-					const Vector3r& tt=YADE_PTR_CAST<CpmState>(b->state)->tau;
-					float s[3]={ss[0],ss[1],ss[2]};
-					float t[3]={tt[0],tt[1],tt[2]};
-					cpmSigma->InsertNextTupleValue(s);
-					cpmSigmaM->InsertNextValue((ss[0]+ss[1]+ss[2])/3.);
-					cpmTau->InsertNextTupleValue(t);
+					const Matrix3r& ss=YADE_PTR_CAST<CpmState>(b->state)->stressTimesV;
+					//float s[3]={ss[0],ss[1],ss[2]};
+					float s[9]={ss[0,0],ss[0,1],ss[0,2],ss[1,0],ss[1,1],ss[1,2],ss[2,0],ss[2,1],ss[2,2]};
+					cpmStress->InsertNextTupleValue(s);
 				}
 				if (recActive[REC_RPM]){
 					rpmSpecNum->InsertNextValue(YADE_PTR_CAST<RpmState>(b->state)->specimenNumber);
@@ -372,9 +379,7 @@ void VTKRecorder::action(){
 		}
 		if (recActive[REC_CPM]){
 			spheresUg->GetPointData()->AddArray(cpmDamage);
-			spheresUg->GetPointData()->AddArray(cpmSigma);
-			spheresUg->GetPointData()->AddArray(cpmSigmaM);
-			spheresUg->GetPointData()->AddArray(cpmTau);
+			spheresUg->GetPointData()->AddArray(cpmStress);
 		}
 		if (recActive[REC_RPM]){
 			spheresUg->GetPointData()->AddArray(rpmSpecNum);
@@ -427,6 +432,9 @@ void VTKRecorder::action(){
 		intrPd->SetLines(intrCells);
 		intrPd->GetCellData()->AddArray(intrForceN);
 		intrPd->GetCellData()->AddArray(intrAbsForceT);
+		if (recActive[REC_WPM]){
+			intrPd->GetCellData()->AddArray(wpmLimitNormalFactor);
+		}
 		#ifdef YADE_VTK_MULTIBLOCK
 			if(!multiblock)
 		#endif
@@ -456,3 +464,6 @@ void VTKRecorder::action(){
 		}
 	#endif
 }
+
+
+#endif /* YADE_VTK */
