@@ -7,8 +7,8 @@
 #include<sstream>
 #include<iostream>
 
-#include<yade/lib-base/Math.hpp>
-#include<yade/lib-pyutil/doc_opts.hpp>
+#include<yade/lib/base/Math.hpp>
+#include<yade/lib/pyutil/doc_opts.hpp>
 
 
 namespace bp=boost::python;
@@ -66,7 +66,6 @@ struct Vector3i_pickle: bp::pickle_suite{static bp::tuple getinitargs(const Vect
 struct Vector2r_pickle: bp::pickle_suite{static bp::tuple getinitargs(const Vector2r& x){ return bp::make_tuple(x[0],x[1]);} };
 struct Vector2i_pickle: bp::pickle_suite{static bp::tuple getinitargs(const Vector2i& x){ return bp::make_tuple(x[0],x[1]);} };
 
-#undef IDX_CHECK
 
 /* template to define custom converter from sequence/list or approriate length and type, to eigen's Vector
    - length is stored in VT::RowsAtCompileTime
@@ -88,6 +87,8 @@ static Matrix3r* Matrix3r_fromElements(Real m00, Real m01, Real m02, Real m10, R
 static Vector6r* Vector6r_fromElements(Real v0, Real v1, Real v2, Real v3, Real v4, Real v5){ Vector6r* v(new Vector6r); (*v)<<v0,v1,v2,v3,v4,v5; return v; }
 static Vector6i* Vector6i_fromElements(int v0, int v1, int v2, int v3, int v4, int v5){ Vector6i* v(new Vector6i); (*v)<<v0,v1,v2,v3,v4,v5; return v; }
 static Vector3r Matrix3r_diagonal(const Matrix3r& m){ return Vector3r(m.diagonal()); }
+static Vector3r Matrix3r_row(const Matrix3r& m, int ix){ IDX_CHECK(ix,3); return Vector3r(m.row(ix)); }
+static Vector3r Matrix3r_col(const Matrix3r& m, int ix){ IDX_CHECK(ix,3); return Vector3r(m.col(ix)); }
 static Vector6r Matrix3r_toVoigt(const Matrix3r& m, bool strain=false){ return tensor_toVoigt(m,strain); }
 static Matrix3r Vector6r_toSymmTensor(const Vector6r& v, bool strain=false){ return voigt_toSymmTensor(v,strain); }
 static Quaternionr Quaternionr_setFromTwoVectors(Quaternionr& q, const Vector3r& u, const Vector3r& v){ return q.setFromTwoVectors(u,v); }
@@ -99,15 +100,23 @@ static bp::tuple Quaternionr_toAxisAngle(const Quaternionr& self){ AngleAxisr aa
 static bp::tuple Quaternionr_toAngleAxis(const Quaternionr& self){ AngleAxisr aa(self); return bp::make_tuple(aa.angle(),aa.axis());}
 
 static Real Vector3r_dot(const Vector3r& self, const Vector3r& v){ return self.dot(v); }
+static Matrix3r Vector3r_outer(const Vector3r& self, const Vector3r& v){ return self*v.transpose(); }
 static Real Vector3i_dot(const Vector3i& self, const Vector3i& v){ return self.dot(v); }
 static Real Vector2r_dot(const Vector2r& self, const Vector2r& v){ return self.dot(v); }
 static Real Vector2i_dot(const Vector2i& self, const Vector2i& v){ return self.dot(v); }
 static Vector3r Vector3r_cross(const Vector3r& self, const Vector3r& v){ return self.cross(v); }
 static Vector3i Vector3i_cross(const Vector3i& self, const Vector3i& v){ return self.cross(v); }
+static Vector3r Vector6r_head(const Vector6r& self){ return self.start<3>(); }
+static Vector3r Vector6r_tail(const Vector6r& self){ return self.end<3>(); }
+static Vector3i Vector6i_head(const Vector6i& self){ return self.start<3>(); }
+static Vector3i Vector6i_tail(const Vector6i& self){ return self.end<3>(); }
 static bool Quaternionr__eq__(const Quaternionr& q1, const Quaternionr& q2){ return q1==q2; }
 static bool Quaternionr__neq__(const Quaternionr& q1, const Quaternionr& q2){ return q1!=q2; }
 #include<Eigen/SVD>
 static bp::tuple Matrix3r_polarDecomposition(const Matrix3r& self){ Matrix3r unitary,positive; Matrix_computeUnitaryPositive(self,&unitary,&positive); return bp::make_tuple(unitary,positive); }
+static bp::tuple Matrix3r_eigenDecomposition(const Matrix3r& self){ Matrix3r rot,diag; matrixEigenDecomposition(self,rot,diag); return bp::make_tuple(rot,Vector3r(diag.diagonal())); }
+
+#undef IDX_CHECK
 
 
 #define EIG_WRAP_METH1(klass,meth) static klass klass##_##meth(const klass& self){ return self.meth(); }
@@ -192,6 +201,7 @@ BOOST_PYTHON_MODULE(miniEigen){
 
 	bp::class_<Matrix3r>("Matrix3","3x3 float matrix.\n\nSupported operations (``m`` is a Matrix3, ``f`` if a float/int, ``v`` is a Vector3): ``-m``, ``m+m``, ``m+=m``, ``m-m``, ``m-=m``, ``m*f``, ``f*m``, ``m*=f``, ``m/f``, ``m/=f``, ``m*m``, ``m*=m``, ``m*v``, ``v*m``, ``m==m``, ``m!=m``.",bp::init<>())
 		.def(bp::init<Matrix3r const &>((bp::arg("m"))))
+		.def(bp::init<Quaternionr const &>((bp::arg("q"))))
 		.def("__init__",bp::make_constructor(&Matrix3r_fromElements,bp::default_call_policies(),(bp::arg("m00"),bp::arg("m01"),bp::arg("m02"),bp::arg("m10"),bp::arg("m11"),bp::arg("m12"),bp::arg("m20"),bp::arg("m21"),bp::arg("m22"))))
 		.def_pickle(Matrix3r_pickle())
 		//
@@ -200,7 +210,10 @@ BOOST_PYTHON_MODULE(miniEigen){
 		.def("inverse",&Matrix3r_inverse)
 		.def("transpose",&Matrix3r_transpose)
 		.def("polarDecomposition",&Matrix3r_polarDecomposition)
+		.def("eigenDecomposition",&Matrix3r_eigenDecomposition)
 		.def("diagonal",&Matrix3r_diagonal)
+		.def("row",&Matrix3r_row)
+		.def("col",&Matrix3r_col)
 
 		//
 		.def("__neg__",&Matrix3r__neg__)
@@ -228,6 +241,7 @@ BOOST_PYTHON_MODULE(miniEigen){
 		.def("__init__",bp::make_constructor(&Quaternionr_fromAxisAngle,bp::default_call_policies(),(bp::arg("axis"),bp::arg("angle"))))
 		.def("__init__",bp::make_constructor(&Quaternionr_fromAngleAxis,bp::default_call_policies(),(bp::arg("angle"),bp::arg("axis"))))
 		.def(bp::init<Real,Real,Real,Real>((bp::arg("w"),bp::arg("x"),bp::arg("y"),bp::arg("z")),"Initialize from coefficients.\n\n.. note:: The order of coefficients is *w*, *x*, *y*, *z*. The [] operator numbers them differently, 0…4 for *x* *y* *z* *w*!"))
+		.def(bp::init<Matrix3r>((bp::arg("rotMatrix")))) //,"Initialize from given rotation matrix.")
 		.def(bp::init<Quaternionr>((bp::arg("other"))))
 		.def_pickle(Quaternionr_pickle())
 		// properties
@@ -262,6 +276,7 @@ BOOST_PYTHON_MODULE(miniEigen){
 		// methods
 		//.def("dot",&Vector6r_dot).def("cross",&Vector6r_cross)
 		.def("norm",&Vector6r::norm).def("squaredNorm",&Vector6r::squaredNorm).def("normalize",&Vector6r::normalize).def("normalized",&Vector6r::normalized)
+		.def("head",&Vector6r_head).def("tail",&Vector6r_tail)
 		// operators
 		.def("__neg__",&Vector6r__neg__) // -v
 		.def("__add__",&Vector6r__add__Vector6r).def("__iadd__",&Vector6r__iadd__Vector6r) // +, +=
@@ -289,6 +304,7 @@ BOOST_PYTHON_MODULE(miniEigen){
 		// methods
 		//.def("dot",&Vector6i_dot).def("cross",&Vector6i_cross)
 		.def("norm",&Vector6i::norm).def("squaredNorm",&Vector6i::squaredNorm).def("normalize",&Vector6i::normalize).def("normalized",&Vector6i::normalized)
+		.def("head",&Vector6i_head).def("tail",&Vector6i_tail)
 		// operators
 		.def("__neg__",&Vector6i__neg__) // -v
 		.def("__add__",&Vector6i__add__Vector6i).def("__iadd__",&Vector6i__iadd__Vector6i) // +, +=
@@ -311,6 +327,7 @@ BOOST_PYTHON_MODULE(miniEigen){
 		.add_static_property("UnitX",&Vector3r_UnitX).add_static_property("UnitY",&Vector3r_UnitY).add_static_property("UnitZ",&Vector3r_UnitZ)
 		// methods
 		.def("dot",&Vector3r_dot).def("cross",&Vector3r_cross)
+		.def("outer",&Vector3r_outer)
 		.def("norm",&Vector3r::norm).def("squaredNorm",&Vector3r::squaredNorm).def("normalize",&Vector3r::normalize).def("normalized",&Vector3r::normalized)
 		// operators
 		.def("__neg__",&Vector3r__neg__) // -v
