@@ -6,14 +6,14 @@ Creating packings and filling volumes defined by boundary representation or cons
 
 For examples, see
 
-* :ysrc:`scripts/test/gts-horse.py`
 * :ysrc:`scripts/test/gts-operators.py`
 * :ysrc:`scripts/test/gts-random-pack-obb.py`
 * :ysrc:`scripts/test/gts-random-pack.py`
 * :ysrc:`scripts/test/pack-cloud.py`
 * :ysrc:`scripts/test/pack-predicates.py`
-* :ysrc:`examples/regular-sphere-pack/regular-sphere-pack.py`
-
+* :ysrc:`examples/packs/packs.py`
+* :ysrc:`examples/gts-horse/gts-horse.py`
+* :ysrc:`examples/WireMatPM/wirepackings.py`
 """
 
 import itertools,warnings
@@ -45,7 +45,9 @@ from _packPredicates import * ## imported in randomDensePack as well
 # import SpherePack
 from _packSpheres import *
 from _packObb import *
-from _packSpherePadder import *
+try:
+	from _packSpherePadder import *
+except ImportError: pass
 
 ##
 # extend _packSphere.SpherePack c++ class by this method
@@ -73,14 +75,19 @@ Add generated packing to the simulation, rotated by 45° along +z
 	>>> sp.toSimulation(rot=Quaternion((0,0,1),pi/4),color=(0,0,1))
 	[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19]
 
-Periodic properties are transferred to the simulation correctly:
+Periodic properties are transferred to the simulation correctly, including rotation:
 
 	>>> O.periodic
 	True
 	>>> O.cell.refSize
 	Vector3(5,5,5)
+	>>> O.cell.hSize
+	Matrix3(3.53553,-3.53553,0, 3.53553,3.53553,0, 0,0,5)
+
+The current state (even if rotated) is taken as mechanically undeformed, i.e. with identity transformation:
+
 	>>> O.cell.trsf
-	Matrix3(0.707107,-0.707107,0, 0.707107,0.707107,0, 0,0,1)
+	Matrix3(1,0,0, 0,1,0, 0,0,1)
 
 :param Quaternion/Matrix3 rot: rotation of the packing, which will be applied on spheres and will be used to set :yref:`Cell.trsf` as well.
 :param **kw: passed to :yref:`yade.utils.sphere`
@@ -90,8 +97,8 @@ Periodic properties are transferred to the simulation correctly:
 	assert(isinstance(rot,Matrix3))
 	if self.cellSize!=Vector3.Zero:
 		O.periodic=True
-		O.cell.trsf=rot
-		O.cell.refSize=self.cellSize
+		O.cell.hSize=rot*Matrix3(self.cellSize[0],0,0, 0,self.cellSize[0],0, 0,0,self.cellSize[1])
+		O.cell.trsf=Matrix3.Identity
 	if not self.hasClumps():
 		return O.bodies.append([utils.sphere(rot*c,r,**kw) for c,r in self])
 	else:
@@ -169,7 +176,7 @@ class inSpace(Predicate):
 def gtsSurface2Facets(surf,**kw):
 	"""Construct facets from given GTS surface. **kw is passed to utils.facet."""
 	import gts
-	return [utils.facet([v.coords() for v in face.vertices()],**kw) for face in surf]
+	return [utils.facet([v.coords() for v in face.vertices()],**kw) for face in surf.faces()]
 
 def sweptPolylines2gtsSurface(pts,threshold=0,capStart=False,capEnd=False):
 	"""Create swept suface (as GTS triangulation) given same-length sequences of points (as 3-tuples).
@@ -271,19 +278,29 @@ def regularHexa(predicate,radius,gap,**kw):
 		if predicate((x,y,z),radius): ret+=[utils.sphere((x,y,z),radius=radius,**kw)]
 	return ret
 
-def filterSpherePack(predicate,spherePack,**kw):
+def filterSpherePack(predicate,spherePack,returnSpherePack=None,**kw):
 	"""Using given SpherePack instance, return spheres the satisfy predicate.
 	The packing will be recentered to match the predicate and warning is given if the predicate
 	is larger than the packing."""
+	if returnSpherePack==None:
+		warnings.warn('The default behavior will change; specify returnSpherePack=True for the new behavior, and False to get rid of this warning (your code will break in the future, however). The returned SpherePack object can be added to the simulation using SpherePack.toSimulation()',category=FutureWarning)
+		returnSpherePack=False
 	mn,mx=predicate.aabb()
 	dimP,centP=predicate.dim(),predicate.center()
 	dimS,centS=spherePack.dim(),spherePack.center()
 	if dimP[0]>dimS[0] or dimP[1]>dimS[1] or dimP[2]>dimS[2]: warnings.warn("Packing's dimension (%s) doesn't fully contain dimension of the predicate (%s)."%(dimS,dimP))
 	spherePack.translate(centP-centS)
-	ret=[]
-	for s in spherePack:
-		if predicate(s[0],s[1]): ret+=[utils.sphere(s[0],radius=s[1],**kw)]
-	return ret
+	if returnSpherePack:
+		ret=SpherePack()
+		for c,r in spherePack:
+			if predicate(c,r): ret.add(c,r)
+		return ret
+	else:
+		# return particles to be added to O.bodies
+		ret=[]
+		for s in spherePack:
+			if predicate(s[0],s[1]): ret+=[utils.sphere(s[0],radius=s[1],**kw)]
+		return ret
 
 def _memoizePacking(memoizeDb,sp,radius,rRelFuzz,wantPeri,fullDim):
 	if not memoizeDb: return
@@ -351,9 +368,9 @@ def _getMemoizedPacking(memoizeDb,radius,rRelFuzz,x1,y1,z1,fullDim,wantPeri,fill
 
 
 
-def randomDensePack(predicate,radius,material=-1,dim=None,cropLayers=0,rRelFuzz=0.,spheresInCell=0,memoizeDb=None,useOBB=True,memoDbg=False,color=None):
+def randomDensePack(predicate,radius,material=-1,dim=None,cropLayers=0,rRelFuzz=0.,spheresInCell=0,memoizeDb=None,useOBB=True,memoDbg=False,color=None,returnSpherePack=None):
 	"""Generator of random dense packing with given geometry properties, using TriaxialTest (aperiodic)
-	or PeriIsoCompressor (periodic). The priodicity depens on whether	the spheresInCell parameter is given.
+	or PeriIsoCompressor (periodic). The periodicity depens on whether	the spheresInCell parameter is given.
 
 	*O.switchScene()* magic is used to have clean simulation for TriaxialTest without deleting the original simulation.
 	This function therefore should never run in parallel with some code accessing your simulation.
@@ -375,6 +392,7 @@ def randomDensePack(predicate,radius,material=-1,dim=None,cropLayers=0,rRelFuzz=
 		computed first; it can reduce substantially number of spheres for the triaxial compression (like 10× depending on
 		how much asymmetric the body is), see scripts/test/gts-triax-pack-obb.py.
 	:param memoDbg: show packigns that are considered and reasons why they are rejected/accepted
+	:param returnSpherePack: see :yref:`filterSpherePack`
 
 	:return: SpherePack object with spheres, filtered by the predicate.
 	"""
@@ -409,7 +427,7 @@ def randomDensePack(predicate,radius,material=-1,dim=None,cropLayers=0,rRelFuzz=
 			if orientation:
 				sp.cellSize=(0,0,0) # resetting cellSize avoids warning when rotating
 				sp.rotate(*orientation.toAxisAngle())
-			return filterSpherePack(predicate,sp,material=material)
+			return filterSpherePack(predicate,sp,material=material,returnSpherePack=returnSpherePack)
 		else: print "No suitable packing in database found, running",'PERIODIC compression' if wantPeri else 'triaxial'
 		sys.stdout.flush()
 	O.switchScene(); O.resetThisScene() ### !!
@@ -417,12 +435,12 @@ def randomDensePack(predicate,radius,material=-1,dim=None,cropLayers=0,rRelFuzz=
 		# x1,y1,z1 already computed above
 		sp=SpherePack()
 		O.periodic=True
-		O.cell.refSize=Vector3(x1,y1,z1)
+		O.cell.refSize=(x1,y1,z1)
 		#print cloudPorosity,beta,gamma,N100,x1,y1,z1,O.cell.refSize
 		#print x1,y1,z1,radius,rRelFuzz
 		O.materials.append(FrictMat(young=3e10,density=2400))
 		num=sp.makeCloud(Vector3().Zero,O.cell.refSize,radius,rRelFuzz,spheresInCell,True)
-		O.engines=[ForceResetter(),InsertionSortCollider([Bo1_Sphere_Aabb()],nBins=5,sweepLength=.05*radius),InteractionLoop([Ig2_Sphere_Sphere_Dem3DofGeom()],[Ip2_FrictMat_FrictMat_FrictPhys()],[Law2_Dem3DofGeom_FrictPhys_CundallStrack()]),PeriIsoCompressor(charLen=2*radius,stresses=[-100e9,-1e8],maxUnbalanced=1e-2,doneHook='O.pause();',globalUpdateInt=5,keepProportions=True),NewtonIntegrator(damping=.6)]
+		O.engines=[ForceResetter(),InsertionSortCollider([Bo1_Sphere_Aabb()],nBins=5,verletDist=.05*radius),InteractionLoop([Ig2_Sphere_Sphere_Dem3DofGeom()],[Ip2_FrictMat_FrictMat_FrictPhys()],[Law2_Dem3DofGeom_FrictPhys_CundallStrack()]),PeriIsoCompressor(charLen=2*radius,stresses=[-100e9,-1e8],maxUnbalanced=1e-2,doneHook='O.pause();',globalUpdateInt=5,keepProportions=True),NewtonIntegrator(damping=.6)]
 		O.materials.append(FrictMat(young=30e9,frictionAngle=.5,poisson=.3,density=1e3))
 		for s in sp: O.bodies.append(utils.sphere(s[0],s[1]))
 		O.dt=utils.PWaveTimeStep()
@@ -448,7 +466,7 @@ def randomDensePack(predicate,radius,material=-1,dim=None,cropLayers=0,rRelFuzz=
 	if orientation:
 		sp.cellSize=(0,0,0); # reset periodicity to avoid warning when rotating periodic packing
 		sp.rotate(*orientation.toAxisAngle())
-	return filterSpherePack(predicate,sp,material=material,color=color)
+	return filterSpherePack(predicate,sp,material=material,color=color,returnSpherePack=returnSpherePack)
 
 def randomPeriPack(radius,initSize,rRelFuzz=0.0,memoizeDb=None):
 	"""Generate periodic dense packing.
@@ -468,9 +486,9 @@ def randomPeriPack(radius,initSize,rRelFuzz=0.0,memoizeDb=None):
 	O.switchScene(); O.resetThisScene()
 	sp=SpherePack()
 	O.periodic=True
-	O.cell.refSize=Vector3(initSize)
+	O.cell.refSize=initSize
 	sp.makeCloud(Vector3().Zero,O.cell.refSize,radius,rRelFuzz,-1,True)
-	O.engines=[ForceResetter(),InsertionSortCollider([Bo1_Sphere_Aabb()],nBins=2,sweepLength=.05*radius),InteractionLoop([Ig2_Sphere_Sphere_Dem3DofGeom()],[Ip2_FrictMat_FrictMat_FrictPhys()],[Law2_Dem3DofGeom_FrictPhys_CundallStrack()]),PeriIsoCompressor(charLen=2*radius,stresses=[-100e9,-1e8],maxUnbalanced=1e-2,doneHook='O.pause();',globalUpdateInt=20,keepProportions=True),NewtonIntegrator(damping=.8)]
+	O.engines=[ForceResetter(),InsertionSortCollider([Bo1_Sphere_Aabb()],nBins=2,verletDist=.05*radius),InteractionLoop([Ig2_Sphere_Sphere_Dem3DofGeom()],[Ip2_FrictMat_FrictMat_FrictPhys()],[Law2_Dem3DofGeom_FrictPhys_CundallStrack()]),PeriIsoCompressor(charLen=2*radius,stresses=[-100e9,-1e8],maxUnbalanced=1e-2,doneHook='O.pause();',globalUpdateInt=20,keepProportions=True),NewtonIntegrator(damping=.8)]
 	O.materials.append(FrictMat(young=30e9,frictionAngle=.1,poisson=.3,density=1e3))
 	for s in sp: O.bodies.append(utils.sphere(s[0],s[1]))
 	O.dt=utils.PWaveTimeStep()
@@ -482,3 +500,93 @@ def randomPeriPack(radius,initSize,rRelFuzz=0.0,memoizeDb=None):
 	O.switchScene()
 	return ret
 
+def hexaNet( radius, cornerCoord=[0,0,0], xLength=1., yLength=0.5, mos=0.08, a=0.04, b=0.04, startAtCorner=True, isSymmetric=False, **kw ):
+	"""Definition of the particles for a hexagonal wire net in the x-y-plane for the WireMatPM.
+
+	:param radius: radius of the particle
+	:param cornerCoord: coordinates of the lower left corner of the net
+	:param xLenght: net length in x-direction
+	:param yLenght: net length in y-direction
+	:param mos: mesh opening size (horizontal distance between the double twists)
+	:param a: length of double-twist 
+	:param b: height of single wire section
+	:param startAtCorner: if true the generation starts with a double-twist at the lower left corner
+	:param isSymmetric: defines if the net is symmetric with respect to the y-axis
+
+	:return: set of spheres which defines the net (net) and exact dimensions of the net (lx,ly).
+	
+	note::
+	This packing works for the WireMatPM only. The particles at the corner are always generated first. For examples on how to use this packing see examples/WireMatPM. In order to create the proper interactions for the net the interaction radius has to be adapted in the simulation.
+
+	"""
+	# check input dimension
+	if(xLength<mos): raise ValueError("xLength must be greater than mos!");
+	if(yLength<2*a+b): raise ValueError("yLength must be greater than 2*a+b!");
+	xstart = cornerCoord[0]
+	ystart = cornerCoord[1]
+	z = cornerCoord[2]
+	ab = a+b
+	# number of double twisted sections in y-direction and real length ly
+	ny = int( (yLength-a)/ab ) + 1
+	ly = ny*a+(ny-1)*b
+	jump=0
+	# number of sections in x-direction and real length lx
+	if isSymmetric:
+		nx = int( xLength/mos ) + 1
+		lx = (nx-1)*mos
+		if not startAtCorner:
+			nx+=-1
+	else:
+		nx = int( (xLength-0.5*mos)/mos ) + 1
+		lx = (nx-1)*mos+0.5*mos
+	net = []
+	# generate corner particles
+	if startAtCorner:
+		if (ny%2==0): # if ny even no symmetry in y-direction
+			net+=[utils.sphere((xstart,ystart+ly,z),radius=radius,**kw)] # upper left corner
+			if isSymmetric:
+				net+=[utils.sphere((xstart+lx,ystart+ly,z),radius=radius,**kw)] # upper right corner
+			else:
+				net+=[utils.sphere((xstart+lx,ystart,z),radius=radius,**kw)] # lower right corner
+		else: # if ny odd symmetry in y-direction
+			if not isSymmetric:
+				net+=[utils.sphere((xstart+lx,ystart,z),radius=radius,**kw)] # lower right corner
+				net+=[utils.sphere((xstart+lx,ystart+ly,z),radius=radius,**kw)] # upper right corner
+		jump=1
+	else: # do not start at corner
+		if (ny%2==0): # if ny even no symmetry in y-direction
+			net+=[utils.sphere((xstart,ystart,z),radius=radius,**kw)] # lower left corner
+			if isSymmetric:
+				net+=[utils.sphere((xstart+lx,ystart,z),radius=radius,**kw)] # lower right corner
+			else:
+				net+=[utils.sphere((xstart+lx,ystart+ly,z),radius=radius,**kw)] # upper right corner
+		else: # if ny odd symmetry in y-direction
+			net+=[utils.sphere((xstart,ystart,z),radius=radius,**kw)] # lower left corner
+			net+=[utils.sphere((xstart,ystart+ly,z),radius=radius,**kw)] # upper left corner
+			if isSymmetric:
+				net+=[utils.sphere((xstart+lx,ystart,z),radius=radius,**kw)] # lower right corner
+				net+=[utils.sphere((xstart+lx,ystart+ly,z),radius=radius,**kw)] # upper right corner
+		xstart+=0.5*mos
+	# generate other particles
+	if isSymmetric:
+		for i in range(ny):
+			y = ystart + i*ab
+			for j in range(nx):
+				x = xstart + j*mos
+				# add two particles of one vertical section (double-twist)
+				net+=[utils.sphere((x,y,z),radius=radius,**kw)]
+				net+=[utils.sphere((x,y+a,z),radius=radius,**kw)]
+			# set values for next section
+			xstart = xstart - 0.5*mos*pow(-1,i+jump)
+			nx = int(nx + 1*pow(-1,i+jump))
+	else:
+		for i in range(ny):
+			y = ystart + i*ab
+			for j in range(nx):
+				x = xstart + j*mos
+				# add two particles of one vertical section (double-twist)
+				net+=[utils.sphere((x,y,z),radius=radius,**kw)]
+				net+=[utils.sphere((x,y+a,z),radius=radius,**kw)]
+			# set values for next section
+			xstart = xstart - 0.5*mos*pow(-1,i+jump)
+	return [net,lx,ly]
