@@ -4,11 +4,11 @@
 import unittest,inspect
 
 # add any new test suites to the list here, so that they are picked up by testAll
-allTests=['wrapper','omega','pbc','clump'] # ,'cohesive-chain']
+allTests=['wrapper','core','pbc','clump','cohesive-chain']
 
 # all yade modules (ugly...)
-import yade.eudoxos,yade.export,yade.linterpolation,yade.log,yade.pack,yade.plot,yade.post2d,yade.timing,yade.utils,yade.ymport
-allModules=(yade.eudoxos,yade.export,yade.linterpolation,yade.log,yade.pack,yade.plot,yade.post2d,yade.timing,yade.utils,yade.ymport)
+import yade.eudoxos,yade.export,yade.linterpolation,yade.log,yade.pack,yade.plot,yade.post2d,yade.timing,yade.utils,yade.ymport,yade.geom
+allModules=(yade.eudoxos,yade.export,yade.linterpolation,yade.log,yade.pack,yade.plot,yade.post2d,yade.timing,yade.utils,yade.ymport,yade.geom)
 try:
 	import yade.qt
 	allModules+=(yade.qt,)
@@ -19,7 +19,7 @@ allTestsFQ=['yade.tests.'+test for test in allTests]
 
 def testModule(module):
 	"""Run all tests defined in the module specified, return TestResult object 
-	(http://www.python.org/doc/2.6/lib/unittest.html#testresult-objects.html)
+	(http://docs.python.org/library/unittest.html#unittest.TextTestResult)
 	for further processing.
 
 	@param module: fully-qualified module name, e.g. yade.tests.wrapper
