@@ -9,7 +9,7 @@
 Devs: please DO NOT ADD more functions here, it is getting too crowded!
 """
 
-import math,random
+import math,random,doctest,geom,numpy
 from yade import *
 from yade.wrapper import *
 from miniEigen import *
@@ -17,7 +17,7 @@ try: # use psyco if available
 	import psyco
 	psyco.full()
 except ImportError: pass
-import doctest
+
 
 # c++ implementations for performance reasons
 from yade._utils import *
@@ -28,26 +28,24 @@ def saveVars(mark='',loadNow=True,**kw):
 	For example, variables *a*, *b* and *c* are defined. To save them, use::
 
 		>>> from yade import utils
-		>>> utils.saveVars('mark',a=1,b=2,c=3)
-		>>> from yade.params.mark import *
+		>>> utils.saveVars('something',a=1,b=2,c=3)
+		>>> from yade.params.something import *
 		>>> a,b,c
 		(1, 2, 3)
 
 	those variables will be save in the .xml file, when the simulation itself is saved. To recover those variables once the .xml is loaded again, use
 
-		>>> utils.loadVars('mark')
+		>>> utils.loadVars('something')
 
-	and they will be defined in the yade.params.\ *mark* module
-
-	m*==True, variables will be loaded immediately after saving. That effectively makes *\*\*kw* available in builtin namespace.
+	and they will be defined in the yade.params.\ *mark* module. The *loadNow* parameter calls :yref:`yade.utils.loadVars` after saving automatically.
 	"""
 	import cPickle
 	Omega().tags['pickledPythonVariablesDictionary'+mark]=cPickle.dumps(kw)
 	if loadNow: loadVars(mark)
 
 def loadVars(mark=None):
-	"""Load variables from saveVars, which are saved inside the simulation.
-	If mark==None, all save variables are loaded. Otherwise only those with
+	"""Load variables from :yref:`yade.utils.saveVars`, which are saved inside the simulation.
+	If ``mark==None``, all save variables are loaded. Otherwise only those with
 	the mark passed."""
 	import cPickle, types, sys, warnings
 	def loadOne(d,mark=None):
@@ -77,7 +75,7 @@ def loadVars(mark=None):
 def SpherePWaveTimeStep(radius,density,young):
 	r"""Compute P-wave critical timestep for a single (presumably representative) sphere, using formula for P-Wave propagation speed $\Delta t_{c}=\frac{r}{\sqrt{E/\rho}}$.
 	If you want to compute minimum critical timestep for all spheres in the simulation, use :yref:`yade.utils.PWaveTimeStep` instead.
-	
+
 	>>> SpherePWaveTimeStep(1e-3,2400,30e9)
 	2.8284271247461903e-07
 	"""
@@ -85,7 +83,7 @@ def SpherePWaveTimeStep(radius,density,young):
 	return radius/sqrt(young/density)
 
 def randomColor():
-	"Return random Vector3 with each component in interval 0…1 (uniform distribution)"
+	"""Return random Vector3 with each component in interval 0…1 (uniform distribution)"""
 	return Vector3(random.random(),random.random(),random.random())
 
 def typedEngine(name):
@@ -98,12 +96,6 @@ def typedEngine(name):
 	"""
 	return [e for e in Omega().engines if e.__class__.__name__==name][0]
 
-def downCast(obj,newClassName):
-	"""Cast given object to class deriving from the same yade root class and copy all parameters from given object.
-	Obj should be up in the inheritance tree, otherwise some attributes may not be defined in the new class."""
-	return obj.__class__(newClassName,dict([ (key,obj[key]) for key in obj.keys() ]))
-
-
 def defaultMaterial():
 	"""Return default material, when creating bodies with :yref:`yade.utils.sphere` and friends, material is unspecified and there is no shared material defined yet. By default, this function returns::
 
@@ -111,7 +103,7 @@ def defaultMaterial():
 	"""
 	return FrictMat(density=1e3,young=1e7,poisson=.3,frictionAngle=.5,label='defaultMat')
 
-def _commonBodySetup(b,volume,geomInertia,material,noBound=False,resetState=True):
+def _commonBodySetup(b,volume,geomInertia,material,pos,noBound=False,resetState=True,dynamic=None,fixed=False):
 	"""Assign common body parameters."""
 	if isinstance(material,int):
 		if material<0 and len(O.materials)==0: O.materials.append(defaultMaterial());
@@ -124,9 +116,16 @@ def _commonBodySetup(b,volume,geomInertia,material,noBound=False,resetState=True
 	if resetState: b.state=b.mat.newAssocState()
 	mass=volume*b.mat.density
 	b.state.mass,b.state.inertia=mass,geomInertia*b.mat.density
+	b.state.pos=b.state.refPos=pos
 	b.bounded=(not noBound)
+	if dynamic!=None:
+		import warnings
+		warnings.warn('dynamic=%s is deprecated, use fixed=%s instead'%(str(dynamic),str(not dynamic)),category=DeprecationWarning,stacklevel=2)
+		fixed=not dynamic
+	b.state.blockedDOFs=('xyzXYZ' if fixed else '')
 
-def sphere(center,radius,dynamic=True,wire=False,color=None,highlight=False,material=-1,mask=1):
+
+def sphere(center,radius,dynamic=None,fixed=False,wire=False,color=None,highlight=False,material=-1,mask=1):
 	"""Create sphere with given parameters; mass and inertia computed automatically.
 
 	Last assigned material is used by default (*material*=-1), and utils.defaultMaterial() will be used if no material is defined at all.
@@ -184,7 +183,7 @@ def sphere(center,radius,dynamic=True,wire=False,color=None,highlight=False,mate
 
 		>>> import random
 		>>> def matFactory(): return ElastMat(young=1e10*random.random(),density=1e3+1e3*random.random())
-		... 
+		...
 		>>> s3=utils.sphere([0,2,0],1,material=matFactory)
 		>>> s4=utils.sphere([1,2,0],1,material=matFactory)
 
@@ -193,40 +192,36 @@ def sphere(center,radius,dynamic=True,wire=False,color=None,highlight=False,mate
 	b.shape=Sphere(radius=radius,color=color if color else randomColor(),wire=wire,highlight=highlight)
 	V=(4./3)*math.pi*radius**3
 	geomInert=(2./5.)*V*radius**2
-	_commonBodySetup(b,V,Vector3(geomInert,geomInert,geomInert),material)
-	b.state.pos=b.state.refPos=center
-	b.dynamic=dynamic
+	_commonBodySetup(b,V,Vector3(geomInert,geomInert,geomInert),material,pos=center,dynamic=dynamic,fixed=fixed)
 	b.aspherical=False
 	b.mask=mask
 	return b
 
-def box(center,extents,orientation=[1,0,0,0],dynamic=True,wire=False,color=None,highlight=False,material=-1,mask=1):
+def box(center,extents,orientation=[1,0,0,0],dynamic=None,fixed=False,wire=False,color=None,highlight=False,material=-1,mask=1):
 	"""Create box (cuboid) with given parameters.
 
 	:param Vector3 extents: half-sizes along x,y,z axes
-	
+
 	See :yref:`yade.utils.sphere`'s documentation for meaning of other parameters."""
 	b=Body()
 	b.shape=Box(extents=extents,color=color if color else randomColor(),wire=wire,highlight=highlight)
 	V=8*extents[0]*extents[1]*extents[2]
 	geomInert=Vector3(4*(extents[1]**2+extents[2]**2),4*(extents[0]**2+extents[2]**2),4*(extents[0]**2+extents[1]**2))
-	_commonBodySetup(b,V,geomInert,material)
-	b.state.pos=b.state.refPos=center
-	b.dynamic=dynamic
+	_commonBodySetup(b,V,geomInert,material,pos=center,dynamic=dynamic,fixed=fixed)
 	b.mask=mask
 	b.aspherical=True
 	return b
 
 
-def chainedCylinder(begin=Vector3(0,0,0),end=Vector3(1.,0.,0.),radius=0.2,dynamic=True,wire=False,color=None,highlight=False,material=-1,mask=1):
-	"""Create and chain a MinkCylinder with given parameters. This shape is the Minkowski sum of line and sphere.
+def chainedCylinder(begin=Vector3(0,0,0),end=Vector3(1.,0.,0.),radius=0.2,dynamic=None,fixed=False,wire=False,color=None,highlight=False,material=-1,mask=1):
+	"""Create and connect a chainedCylinder with given parameters. The shape generated by repeted calls of this function is the Minkowski sum of polyline and sphere.
 
 	:param Real radius: radius of sphere in the Minkowski sum.
 	:param Vector3 begin: first point positioning the line in the Minkowski sum
 	:param Vector3 last: last point positioning the line in the Minkowski sum
 
 	In order to build a correct chain, last point of element of rank N must correspond to first point of element of rank N+1 in the same chain (with some tolerance, since bounding boxes will be used to create connections.
-	
+
 	:return: Body object with the :yref:`ChainedCylinder` :yref:`shape<Body.shape>`.
 	"""
 	segment=end-begin
@@ -236,20 +231,17 @@ def chainedCylinder(begin=Vector3(0,0,0),end=Vector3(1.,0.,0.),radius=0.2,dynami
 	V=2*(4./3)*math.pi*radius**3
 	geomInert=(2./5.)*V*radius**2+b.shape.length*b.shape.length*2*(4./3)*math.pi*radius**3
 	b.state=ChainedState()
-	b.state.addToChain(b.id)
-	#b.state.blockedDOFs=['rx','ry','rz']
-	_commonBodySetup(b,V,Vector3(geomInert,geomInert,geomInert),material,resetState=False)
-	b.state.pos=b.state.refPos=begin
-	b.dynamic=dynamic
+	b.state.addToChain(O.bodies.append(b))
+	_commonBodySetup(b,V,Vector3(geomInert,geomInert,geomInert),material,pos=begin,resetState=False,dynamic=dynamic,fixed=fixed)
 	b.mask=mask
 	b.bound=Aabb(color=[0,1,0])
 	b.state.ori=b.state.ori.setFromTwoVectors(Vector3(0.,0.,1.),segment)
+	if (end == begin): b.state.ori = Quaternion((1,0,0),0)
 	return b
 
 def wall(position,axis,sense=0,color=None,material=-1,mask=1):
 	"""Return ready-made wall body.
 
-	:Parameters:
 	:param float-or-Vector3 position: center of the wall. If float, it is the position along given axis, the other 2 components being zero
 	:param ∈{0,1,2} axis: orientation of the wall normal (0,1,2) for x,y,z (sc. planes yz, xz, xy)
 	:param ∈{-1,0,1} sense: sense in which to interact (0: both, -1: negative, +1: positive; see :yref:`Wall`)
@@ -257,153 +249,41 @@ def wall(position,axis,sense=0,color=None,material=-1,mask=1):
 	See :yref:`yade.utils.sphere`'s documentation for meaning of other parameters."""
 	b=Body()
 	b.shape=Wall(sense=sense,axis=axis,color=color if color else randomColor())
-	_commonBodySetup(b,0,Vector3(0,0,0),material)
 	if isinstance(position,(int,long,float)):
 		pos2=Vector3(0,0,0); pos2[axis]=position
 	else: pos2=position
-	b.state.pos=b.state.refPos=pos2
-	b.dynamic=False
-	b.aspherical=True # never used, since the wall is not dynamic
+	_commonBodySetup(b,0,Vector3(0,0,0),material,pos=pos2,fixed=True)
+	b.aspherical=False # wall never moves dynamically
 	b.mask=mask
 	return b
 
-def facet(vertices,dynamic=False,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1):
+def facet(vertices,dynamic=None,fixed=True,wire=True,color=None,highlight=False,noBound=False,material=-1,mask=1):
 	"""Create facet with given parameters.
 
-	:Parameters:
 	:param [Vector3,Vector3,Vector3] vertices: coordinates of vertices in the global coordinate system.
 	:param bool wire: if ``True``, facets are shown as skeleton; otherwise facets are filled
 	:param bool noBound: set :yref:`Body.bounded`
 	:param Vector3-or-None color: color of the facet; random color will be assigned if ``None``.
-	
+
 	See :yref:`yade.utils.sphere`'s documentation for meaning of other parameters."""
 	b=Body()
 	center=inscribedCircleCenter(vertices[0],vertices[1],vertices[2])
 	vertices=Vector3(vertices[0])-center,Vector3(vertices[1])-center,Vector3(vertices[2])-center
 	b.shape=Facet(color=color if color else randomColor(),wire=wire,highlight=highlight,vertices=vertices)
-	_commonBodySetup(b,0,Vector3(0,0,0),material,noBound=noBound)
-	b.state.pos=b.state.refPos=center
-	b.dynamic=dynamic
-	b.aspherical=True
+	_commonBodySetup(b,0,Vector3(0,0,0),material,noBound=noBound,pos=center,fixed=fixed)
+	b.aspherical=False # mass and inertia are 0 anyway; fell free to change to ``True`` if needed
 	b.mask=mask
 	return b
+	
+def facetBox(*args,**kw):
+	"|ydeprecated|"
+	_deprecatedUtilsFunction('facetBox','geom.facetBox')
+	return geom.facetBox(*args,**kw)
 
-def facetBox(center,extents,orientation=Quaternion.Identity,wallMask=63,**kw):
-	"""
-	Create arbitrarily-aligned box composed of facets, with given center, extents and orientation.
-	If any of the box dimensions is zero, corresponding facets will not be created. The facets are oriented outwards from the box.
-
-	
-	:Parameters:
-			`center`: Vector3
-				center of the created box
-			`extents`: (eX,eY,eZ)
-				lengths of the box sides
-			`orientation`: quaternion
-				orientation of the box
-			`wallMask`: bitmask
-				determines which walls will be created, in the order -x (1), +x (2), -y (4), +y (8), -z (16), +z (32). The numbers are ANDed; the default 63 means to create all walls;
-			`**kw`: (unused keyword arguments)
-				passed to utils.facet
-
-	:Returns: list of facets forming the box.
-	"""
-	
-	
-	#Defense from zero dimensions
-	if (wallMask>63):
-		print "wallMask must be 63 or less"
-		wallMask=63
-	if (extents[0]==0):
-		wallMask=1
-	elif (extents[1]==0):
-		wallMask=4
-	elif (extents[2]==0):
-		wallMask=16
-	if (((extents[0]==0) and (extents[1]==0)) or ((extents[0]==0) and (extents[2]==0)) or ((extents[1]==0) and (extents[2]==0))):
-		raise RuntimeError("Please, specify at least 2 none-zero dimensions in extents!");
-	# ___________________________
-	
-	mn,mx=[-extents[i] for i in 0,1,2],[extents[i] for i in 0,1,2]
-	def doWall(a,b,c,d):
-		return [facet((a,b,c),**kw),facet((a,c,d),**kw)]
-	ret=[]
-
-	A=orientation*Vector3(mn[0],mn[1],mn[2])+center
-	B=orientation*Vector3(mx[0],mn[1],mn[2])+center
-	C=orientation*Vector3(mx[0],mx[1],mn[2])+center
-	D=orientation*Vector3(mn[0],mx[1],mn[2])+center
-	E=orientation*Vector3(mn[0],mn[1],mx[2])+center
-	F=orientation*Vector3(mx[0],mn[1],mx[2])+center
-	G=orientation*Vector3(mx[0],mx[1],mx[2])+center
-	H=orientation*Vector3(mn[0],mx[1],mx[2])+center
-	if wallMask&1:  ret+=doWall(A,D,H,E)
-	if wallMask&2:  ret+=doWall(B,F,G,C)
-	if wallMask&4:  ret+=doWall(A,E,F,B)
-	if wallMask&8:  ret+=doWall(D,C,G,H)
-	if wallMask&16: ret+=doWall(A,B,C,D)
-	if wallMask&32: ret+=doWall(E,H,G,F)
-	return ret
-	
-def facetCylinder(center,radius,height,orientation=Quaternion.Identity,segmentsNumber=10,wallMask=7,angleRange=None,closeGap=False,**kw):
-	"""
-	Create arbitrarily-aligned cylinder composed of facets, with given center, radius, height and orientation.
-	Return List of facets forming the cylinder;
-	
-	:param Vector3 center: center of the created cylinder
-	:param float radius:  cylinder radius
-	:param float height: cylinder height
-	:param Quaternion orientation: orientation of the cylinder; the reference orientation has axis along the $+x$ axis.
-	:param int segmentsNumber: number of edges on the cylinder surface (>=5)
-	:param bitmask wallMask: determines which walls will be created, in the order up (1), down (2), side (4). The numbers are ANDed; the default 7 means to create all walls
-	:param (θmin,Θmax) angleRange: allows one to create only part of cylinder by specifying range of angles; if ``None``, (0,2*pi) is assumed.
-	:param bool closeGap: close range skipped in angleRange with triangular facets at cylinder bases.
-	:param **kw: (unused keyword arguments) passed to utils.facet;
-	"""
-	
-	# check zero dimentions
-	if (segmentsNumber<3): raise RuntimeError("The segmentsNumber should be at least 3");
-	if (height<=0): raise RuntimeError("The height should have the positive value");
-	if (radius<=0): raise RuntimeError("The radius should have the positive value");
-
-	import numpy
-	if angleRange==None: angleRange=(0,2*math.pi)
-	if isinstance(angleRange,float):
-		print u'WARNING: utils.facetCylinder,angleRange should be (Θmin,Θmax), not just Θmax (one number), update your code.'
-		angleRange=(0,angleRange)
-	anglesInRad = numpy.linspace(angleRange[0], angleRange[1], segmentsNumber+1, endpoint=True)
-	P1=[]; P2=[]
-	P1.append(Vector3(0,0,-height/2))
-	P2.append(Vector3(0,0,+height/2))
-	
-	for i in anglesInRad:
-		X=radius*math.cos(i)
-		Y=radius*math.sin(i)
-		P1.append(Vector3(X,Y,-height/2))
-		P2.append(Vector3(X,Y,+height/2))
-	for i in range(0,len(P1)):
-		P1[i]=orientation*P1[i]+center
-		P2[i]=orientation*P2[i]+center
-		
-	ret=[]
-	for i in range(2,len(P1)):
-		if wallMask&2:
-			ret.append(facet((P1[0],P1[i],P1[i-1]),**kw))
-		if wallMask&1:
-			ret.append(facet((P2[0],P2[i-1],P2[i]),**kw))
-		if wallMask&4:
-			ret.append(facet((P1[i],P2[i],P2[i-1]),**kw))
-			ret.append(facet((P2[i-1],P1[i-1],P1[i]),**kw))
-	if closeGap and (angleRange[0]%(2*math.pi))!=(angleRange[1]%(2*math.pi)): # some part is skipped
-		pts=[(radius*math.cos(angleRange[i]),radius*math.sin(angleRange[i])) for i in (0,1)]
-		for Z in -height/2,height/2:
-			#print (pts[0][0],pts[0][1],Z),(pts[1][0],pts[1][1],Z),(0,0,Z)
-			pp=[(pts[0][0],pts[0][1],Z),(pts[1][0],pts[1][1],Z),(0,0,Z)]
-			pp=[orientation*p+center for p in pp]
-			ret.append(facet(pp,**kw))
-		
-	return ret
-	
+def facetCylinder(*args,**kw):
+	"|ydeprecated|"
+	_deprecatedUtilsFunction('facetCylinder','geom.facetCylinder')
+	return geom.facetCylinder(*args,**kw)
 	
 def aabbWalls(extrema=None,thickness=None,oversizeFactor=1.5,**kw):
 	"""Return 6 boxes that will wrap existing packing as walls from all sides;
@@ -422,7 +302,7 @@ def aabbWalls(extrema=None,thickness=None,oversizeFactor=1.5,**kw):
 		extents[axis]=thickness/2.
 		for j in [0,1]:
 			center[axis]=extrema[j][axis]+(j-.5)*thickness
-			walls.append(box(center=center,extents=extents,dynamic=False,**kw))
+			walls.append(box(center=center,extents=extents,fixed=True,**kw))
 			walls[-1].shape.wire=True
 	return walls
 
@@ -470,7 +350,7 @@ def randomizeColors(onlyDynamic=False):
 		if b.dynamic or not onlyDynamic: b.shape.color=color
 
 def avgNumInteractions(cutoff=0.,skipFree=False):
-	r"""Return average numer of interactions per particle, also known as *coordination number* $Z$. This number is defined as 
+	r"""Return average numer of interactions per particle, also known as *coordination number* $Z$. This number is defined as
 
 	.. math:: Z=2C/N
 
@@ -481,7 +361,9 @@ def avgNumInteractions(cutoff=0.,skipFree=False):
 	.. math:: Z_m=\frac{2C-N_1}{N-N_0-N_1}
 
 	:param cutoff: cut some relative part of the sample's bounding box away.
-	:param skipFree: 	"""
+	:param skipFree: see above.
+	
+"""
 	if cutoff==0 and not skipFree: return 2*O.interactions.countReal()*1./len(O.bodies)
 	else:
 		nums,counts=bodyNumInteractionsHistogram(aabbExtrema(cutoff))
@@ -508,7 +390,7 @@ def plotNumInteractionsHistogram(cutoff=0.):
 def plotDirections(aabb=(),mask=0,bins=20,numHist=True,noShow=False):
 	"""Plot 3 histograms for distribution of interaction directions, in yz,xz and xy planes and
 	(optional but default) histogram of number of interactions per body.
-	
+
 	:returns: If *noShow* is ``False``, displays the figure and returns nothing. If *noShow*, the figure object is returned without being displayed (works the same way as :yref:`yade.plot.plot`).
 	"""
 	import pylab,math
@@ -538,28 +420,30 @@ def encodeVideoFromFrames(*args,**kw):
 	_deprecatedUtilsFunction('utils.encodeVideoFromFrames','utils.makeVideo')
 	return makeVideo(*args,**kw)
 
-def makeVideo(frameSpec,out,renameNotOverwrite=True,fps=24,bps=2400):
+def makeVideo(frameSpec,out,renameNotOverwrite=True,fps=24,kbps=6000,bps=None):
 	"""Create a video from external image files using `mencoder <http://www.mplayerhq.hu>`__. Two-pass encoding using the default mencoder codec (mpeg4) is performed, running multi-threaded with number of threads equal to number of OpenMP threads allocated for Yade.
 
 	:param frameSpec: wildcard | sequence of filenames. If list or tuple, filenames to be encoded in given order; otherwise wildcard understood by mencoder's mf:// URI option (shell wildcards such as ``/tmp/snap-*.png`` or and printf-style pattern like ``/tmp/snap-%05d.png``)
 	:param str out: file to save video into
 	:param bool renameNotOverwrite: if True, existing same-named video file will have -*number* appended; will be overwritten otherwise.
 	:param int fps: Frames per second (``-mf fps=…``)
-	:param int bps: Bitrate (``-lavcopts vbitrate=…``)
+	:param int kbps: Bitrate (``-lavcopts vbitrate=…``) in kb/s
 	"""
-	import os,os.path,subprocess
+	import os,os.path,subprocess,warnings
+	if bps!=None:
+		warnings.warn('plot.makeVideo: bps is deprecated, use kbps instead (the significance is the same, but the name is more precise)',stacklevel=2,category=DeprecationWarning)
+		kbps=bps
 	if renameNotOverwrite and os.path.exists(out):
 		i=0
 		while(os.path.exists(out+"~%d"%i)): i+=1
 		os.rename(out,out+"~%d"%i); print "Output file `%s' already existed, old file renamed to `%s'"%(out,out+"~%d"%i)
 	if isinstance(frameSpec,list) or isinstance(frameSpec,tuple): frameSpec=','.join(frameSpec)
 	for passNo in (1,2):
-		cmd=['mencoder','mf://%s'%frameSpec,'-mf','fps=%d'%int(fps),'-ovc','lavc','-lavcopts','vbitrate=%d:vpass=%d:threads=%d:%s'%(int(bps),passNo,O.numThreads,'turbo' if passNo==1 else ''),'-o',('/dev/null' if passNo==1 else out)]
+		cmd=['mencoder','mf://%s'%frameSpec,'-mf','fps=%d'%int(fps),'-ovc','lavc','-lavcopts','vbitrate=%d:vpass=%d:threads=%d:%s'%(int(kbps),passNo,O.numThreads,'turbo' if passNo==1 else ''),'-o',('/dev/null' if passNo==1 else out)]
 		print 'Pass %d:'%passNo,' '.join(cmd)
 		ret=subprocess.call(cmd)
 		if ret!=0: raise RuntimeError("Error when running mencoder.")
 
-
 def replaceCollider(colliderEngine):
 	"""Replaces collider (Collider) engine with the engine supplied. Raises error if no collider is in engines."""
 	colliderIdx=-1
@@ -589,15 +473,10 @@ def uniaxialTestFeatures(filename=None,areaSections=10,axis=-1,**kw):
 
 #. Find the bodies that are on the negative/positive boundary, to which the straining condition should be applied.
 
-:parameters:
-	`filename`:
-		if given, spheres will be loaded from this file (ASCII format); if not, current simulation will be used.
-	`areaSection`:
-		number of section that will be used to estimate cross-section
-	`axis`:
-		if given, force strained axis, rather than computing it from predominant length
-
-:return: dictionary with keys 'negIds', 'posIds', 'axis', 'area'.
+:param filename: if given, spheres will be loaded from this file (ASCII format); if not, current simulation will be used.
+:param float areaSection: number of section that will be used to estimate cross-section
+:param ∈{0,1,2} axis: if given, force strained axis, rather than computing it from predominant length
+:return: dictionary with keys ``negIds``, ``posIds``, ``axis``, ``area``.
 
 .. warning::
 	The function :yref:`yade.utils.approxSectionArea` uses convex hull algorithm to find the area, but the implementation is reported to be *buggy* (bot works in some cases). Always check this number, or fix the convex hull algorithm (it is documented in the source, see :ysrc:`py/_utils.cpp`).
@@ -614,9 +493,93 @@ def uniaxialTestFeatures(filename=None,areaSections=10,axis=-1,**kw):
 	negIds,posIds=negPosExtremeIds(axis=axis,distFactor=2.2)
 	return {'negIds':negIds,'posIds':posIds,'axis':axis,'area':min(areas)}
 
+def voxelPorosityTriaxial(triax,resolution=200,offset=0):
+	"""
+	Calculate the porosity of a sample, given the TriaxialCompressionEngine.
+
+	A function :yref:`yade.utils.voxelPorosity` is invoked, with the volume of a box enclosed by TriaxialCompressionEngine walls.
+	The additional parameter offset allows using a smaller volume inside the box, where each side of the volume is at offset distance
+	from the walls. By this way it is possible to find a more precise porosity of the sample, since at walls' contact the porosity is usually reduced.
+	
+	A recommended value of offset is bigger or equal to the average radius of spheres inside.
+	
+	The value of resolution depends on size of spheres used. It can be calibrated by invoking voxelPorosityTriaxial with offset=0 and
+	comparing the result with TriaxialCompressionEngine.porosity. After calibration, the offset can be set to radius, or a bigger value, to get
+	the result.
+	
+	:param triax: the TriaxialCompressionEngine handle
+	:param resolution: voxel grid resolution
+	:param offset: offset distance
+	:return: the porosity of the sample inside given volume
+
+	Example invocation::
+	
+		from yade import utils
+		rAvg=0.03
+		TriaxialTest(numberOfGrains=200,radiusMean=rAvg).load()
+		O.dt=-1
+		O.run(1000)
+		O.engines[4].porosity
+		0.44007807740143889
+		utils.voxelPorosityTriaxial(O.engines[4],200,0)
+		0.44055412500000002
+		utils.voxelPorosityTriaxial(O.engines[4],200,rAvg)
+		0.36798199999999998
+	"""
+	p_bottom	= O.bodies[triax.wall_bottom_id].state.se3[0]
+	p_top		= O.bodies[triax.wall_top_id	 ].state.se3[0]
+	p_left		= O.bodies[triax.wall_left_id	].state.se3[0]
+	p_right		= O.bodies[triax.wall_right_id ].state.se3[0]
+	p_front		= O.bodies[triax.wall_front_id ].state.se3[0]
+	p_back		= O.bodies[triax.wall_back_id	].state.se3[0]
+	th							= (triax.thickness)*0.5+offset
+	x_0						 = p_left	[0] + th
+	x_1						 = p_right [0] - th
+	y_0						 = p_bottom[1] + th
+	y_1						 = p_top	 [1] - th
+	z_0						 = p_back	[2] + th
+	z_1						 = p_front [2] - th
+	a=Vector3(x_0,y_0,z_0)
+	b=Vector3(x_1,y_1,z_1)
+	return voxelPorosity(resolution,a,b)
+
+
+def trackPerfomance(updateTime=5):
+	"""
+	Track perfomance of a simulation. (Experimental)
+	Will create new thread to produce some plots.
+	Useful for track perfomance of long run simulations (in bath mode for example).
+	"""
+
+	def __track_perfomance(updateTime):
+		pid=os.getpid()
+		threadsCpu={}
+		lastTime,lastIter=-1,-1
+		while 1:
+			time.sleep(updateTime)
+			if not O.running: 
+				lastTime,lastIter=-1,-1
+				continue
+			if lastTime==-1: 
+				lastTime=time.time();lastIter=O.iter
+				plot.plots.update({'Iteration':('Perfomance',None,'Bodies','Interactions')})
+				continue
+			curTime=time.time();curIter=O.iter
+			perf=(curIter-lastIter)/(curTime-lastTime)
+			out=subprocess.Popen(['top','-bH','-n1', ''.join(['-p',str(pid)])],stdout=subprocess.PIPE).communicate()[0].splitlines()
+			for s in out[7:-1]:
+				w=s.split()
+				threadsCpu[w[0]]=float(w[8])
+			plot.addData(Iteration=curIter,Iter=curIter,Perfomance=perf,Bodies=len(O.bodies),Interactions=len(O.interactions),**threadsCpu)
+			plot.plots.update({'Iter':threadsCpu.keys()})
+			lastTime=time.time();lastIter=O.iter
+
+	thread.start_new_thread(__track_perfomance,(updateTime))
+
+
 def NormalRestitution2DampingRate(en):
 	r"""Compute the normal damping rate as a function of the normal coefficient of restitution $e_n$. For $e_n\in\langle0,1\rangle$ damping rate equals
-	
+
 	.. math:: -\frac{\log e_n}{\sqrt{e_n^2+\pi^2}}
 
 	"""
@@ -644,15 +607,15 @@ def _deprecatedUtilsFunction(old,new):
 # example of _deprecatedUtilsFunction usage:
 #
 # def import_mesh_geometry(*args,**kw):
-#    "|ydeprecated|"
-#    _deprecatedUtilsFunction('import_mesh_geometry','yade.import.gmsh')
-#    import yade.ymport
-#    return yade.ymport.stl(*args,**kw)
+#		"|ydeprecated|"
+#		_deprecatedUtilsFunction('import_mesh_geometry','yade.import.gmsh')
+#		import yade.ymport
+#		return yade.ymport.stl(*args,**kw)
 
 
 class TableParamReader():
 	"""Class for reading simulation parameters from text file.
-	
+
 Each parameter is represented by one column, each parameter set by one line. Colums are separated by blanks (no quoting).
 
 First non-empty line contains column titles (without quotes).
@@ -707,16 +670,16 @@ This class is used by :yref:`yade.utils.readParamsFromTable`.
 				values[l]['description']=dd
 				descs.add(dd)
 		self.values=values
-				
+
 	def paramDict(self):
 		"""Return dictionary containing data from file given to constructor. Keys are line numbers (which might be non-contiguous and refer to real line numbers that one can see in text editors), values are dictionaries mapping parameter names to their values given in the file. The special value '=' has already been interpreted, ``!`` (bangs) (if any) were already removed from column titles, ``description`` column has already been added (if absent)."""
 		return self.values
-		
+
 if __name__=="__main__":
 	tryTable="""head1 important2! !OMP_NUM_THREADS! abcd
 	1 1.1 1.2 1.3
-	'a' 'b' 'c' 'd'  ### comment
-	
+	'a' 'b' 'c' 'd'	### comment
+
 	# empty line
 	1 = = g
 """
@@ -726,62 +689,54 @@ if __name__=="__main__":
 	f.close()
 	from pprint import *
 	pprint(TableParamReader(file).paramDict())
-	
+
 def runningInBatch():
 	'Tell whether we are running inside the batch or separately.'
 	import os
-	return os.environ.has_key('PARAM_TABLE')
+	return 'YADE_BATCH' in os.environ
 
 def waitIfBatch():
 	'Block the simulation if running inside a batch. Typically used at the end of script so that it does not finish prematurely in batch mode (the execution would be ended in such a case).'
 	if runningInBatch(): O.wait()
 
-def readParamsFromTable(tableFileLine=None,noTableOk=False,unknownOk=False,**kw):
+def readParamsFromTable(tableFileLine=None,noTableOk=True,unknownOk=False,**kw):
 	"""
 	Read parameters from a file and assign them to __builtin__ variables.
 
 	The format of the file is as follows (commens starting with # and empty lines allowed)::
-		
+
 		# commented lines allowed anywhere
 		name1 name2 … # first non-blank line are column headings
 					# empty line is OK, with or without comment
-		val1  val2  … # 1st parameter set
-		val2  val2  … # 2nd 
+		val1	val2	… # 1st parameter set
+		val2	val2	… # 2nd
 		…
 
-	Assigned tags:
+	Assigned tags (the ``description`` column is synthesized if absent,see :yref:`yade.utils.TableParamReader`); 
 
-	* *description* column is assigned to Omega().tags['description']; this column is synthesized if absent (see :yref:`yade.utils.TableParamReader`);
-	* ``Omega().tags['params']="name1=val1,name2=val2,…"``
-	* ``Omega().tags['defaultParams']="unassignedName1=defaultValue1,…"``
-	* ``Omega().tags['d.id']=O.tags['id']+'.'+O.tags['description']``
-	* ``Omega().tags['id.d']=O.tags['description']+'.'+O.tags['id']``
+		O.tags['description']=…																			# assigns the description column; might be synthesized
+		O.tags['params']="name1=val1,name2=val2,…"									 # all explicitly assigned parameters
+		O.tags['defaultParams']="unassignedName1=defaultValue1,…"		# parameters that were left at their defaults
+		O.tags['d.id']=O.tags['id']+'.'+O.tags['description']
+		O.tags['id.d']=O.tags['description']+'.'+O.tags['id']
 
 	All parameters (default as well as settable) are saved using :yref:`yade.utils.saveVars`\ ``('table')``.
 
-	:parameters:
-		`tableFile`:
-			text file (with one value per blank-separated columns)
-		`tableLine`:
-			number of line where to get the values from.
-		`noTableOk`: bool
-			do not raise exception if the file cannot be open; use default values
-		`unknownOk`: bool
-			do not raise exception if unknown column name is found in the file; assign it as well
-
-	:return:
-		number of assigned parameters.
-
+	:param tableFile: text file (with one value per blank-separated columns)
+	:param int tableLine: number of line where to get the values from
+	:param bool noTableOk: if False, raise exception if the file cannot be open; use default values otherwise
+	:param bool unknownOk: do not raise exception if unknown column name is found in the file, and assign it as well
+	:return: number of assigned parameters
 	"""
 	tagsParams=[]
 	# dictParams is what eventually ends up in yade.params.table (default+specified values)
 	dictDefaults,dictParams,dictAssign={},{},{}
 	import os, __builtin__,re,math
-	if not tableFileLine and not os.environ.has_key('PARAM_TABLE'):
-		if not noTableOk: raise EnvironmentError("PARAM_TABLE is not defined in the environment")
+	if not tableFileLine and ('YADE_BATCH' not in os.environ or os.environ['YADE_BATCH']==''):
+		if not noTableOk: raise EnvironmentError("YADE_BATCH is not defined in the environment")
 		O.tags['line']='l!'
 	else:
-		if not tableFileLine: tableFileLine=os.environ['PARAM_TABLE']
+		if not tableFileLine: tableFileLine=os.environ['YADE_BATCH']
 		env=tableFileLine.split(':')
 		tableFile,tableLine=env[0],int(env[1])
 		allTab=TableParamReader(tableFile).paramDict()
@@ -794,7 +749,7 @@ def readParamsFromTable(tableFileLine=None,noTableOk=False,unknownOk=False,**kw)
 		# !something cols are skipped, those are env vars we don't treat at all (they are contained in description, though)
 		for col in vv.keys():
 			if col=='description' or col[0]=='!': continue
-			if col not in kw.keys() and (not unknownOk): raise NameError("Parameter `%s' has no default value assigned"%names[i])
+			if col not in kw.keys() and (not unknownOk): raise NameError("Parameter `%s' has no default value assigned"%col)
 			if vv[col]=='*': vv[col]=kw[col] # use default value for * in the table
 			elif col in kw.keys(): kw.pop(col) # remove the var from kw, so that it contains only those that were default at the end of this loop
 			#print 'ASSIGN',col,vv[col]
@@ -804,10 +759,63 @@ def readParamsFromTable(tableFileLine=None,noTableOk=False,unknownOk=False,**kw)
 	defaults=[]
 	for k in kw.keys():
 		dictDefaults[k]=kw[k]
-		defaults+=["%s=%s"%(k,kw[k])]; 
+		defaults+=["%s=%s"%(k,kw[k])];
 	O.tags['defaultParams']=",".join(defaults)
 	O.tags['params']=",".join(tagsParams)
 	dictParams.update(dictDefaults)
 	saveVars('table',loadNow=True,**dictParams)
 	return len(tagsParams)
+	
+def psd(bins=5, mass=True, mask=-1):
+	"""Calculates particle size distribution.
+	
+	:param int bins: number of bins
+	:param bool mass: if true, the mass-PSD will be calculated
+	:param int mask: :yref:`Body.mask` for the body
+	:return:
+		* binsSizes: list of bin's sizes
+		* binsProc: how much material (in procents) are in the bin, cumulative
+		* binsSumCum: how much material (in units) are in the bin, cumulative
+		binsSizes, binsProc, binsSumCum
+	"""
+	maxD = 0.0
+	minD = 0.0
+	
+	for b in O.bodies:
+		if (isinstance(b.shape,Sphere) and ((mask<0) or ((b.mask&mask)<>0))):
+			if ((2*b.shape.radius)	> maxD) : maxD = 2*b.shape.radius
+			if (((2*b.shape.radius)	< minD) or (minD==0.0)): minD = 2*b.shape.radius
 
+	if (minD==maxD): return false       #All particles are having the same size
+  
+	binsSizes = numpy.linspace(minD, maxD, bins+1)
+	
+	deltaBinD = (maxD-minD)/bins
+	binsMass = numpy.zeros(bins)
+	binsNumbers = numpy.zeros(bins)
+	
+		
+	for b in O.bodies:
+		if (isinstance(b.shape,Sphere) and ((mask<0) or ((b.mask&mask)<>0))):
+			d=2*b.shape.radius
+			
+			basketId = int(math.floor( (d-minD) / deltaBinD ) )
+			if (d == maxD): basketId = bins-1												 #If the diametr equals the maximal diameter, put the particle into the last bin
+			binsMass[basketId] = binsMass[basketId] + b.state.mass		#Put masses into the bin 
+			binsNumbers[basketId] = binsNumbers[basketId] + 1					#Put numbers into the bin 
+			
+	binsProc = numpy.zeros(bins+1)
+	binsSumCum = numpy.zeros(bins+1)
+	
+	i=1
+	for size in binsSizes[:-1]:
+		if (mass): binsSumCum[i]+=binsSumCum[i-1]+binsMass[i-1]		#Calculate mass
+		else: binsSumCum[i]+=binsSumCum[i-1]+binsNumbers[i-1]			#Calculate number of particles
+		i+=1
+	
+	if (binsSumCum[len(binsSumCum)-1] > 0):
+		i=0
+		for l in binsSumCum:
+			binsProc[i] = binsSumCum[i]/binsSumCum[len(binsSumCum)-1]
+			i+=1
+	return binsSizes, binsProc, binsSumCum
