@@ -24,28 +24,28 @@
 #include<boost/algorithm/string.hpp>
 #include<boost/version.hpp>
 
-#include<yade/lib-base/Logging.hpp>
-#include<yade/lib-pyutil/gil.hpp>
-#include<yade/lib-pyutil/raw_constructor.hpp>
-#include<yade/lib-pyutil/doc_opts.hpp>
+#include<yade/lib/base/Logging.hpp>
+#include<yade/lib/pyutil/gil.hpp>
+#include<yade/lib/pyutil/raw_constructor.hpp>
+#include<yade/lib/pyutil/doc_opts.hpp>
 #include<yade/core/Omega.hpp>
 #include<yade/core/ThreadRunner.hpp>
 #include<yade/core/FileGenerator.hpp>
 #include<yade/core/EnergyTracker.hpp>
 
-#include<yade/pkg-dem/STLImporter.hpp>
+#include<yade/pkg/dem/STLImporter.hpp>
 
-#include<yade/pkg-common/Dispatching.hpp>
+#include<yade/pkg/common/Dispatching.hpp>
 #include<yade/core/GlobalEngine.hpp>
 #include<yade/core/PartialEngine.hpp>
 #include<yade/core/Functor.hpp>
-#include<yade/pkg-common/ParallelEngine.hpp>
-#include<yade/pkg-common/Collider.hpp>
+#include<yade/pkg/common/ParallelEngine.hpp>
+#include<yade/pkg/common/Collider.hpp>
 
-#include<yade/pkg-common/InteractionLoop.hpp>
+#include<yade/pkg/common/InteractionLoop.hpp>
 
-// #include<yade/pkg-dem/Shop.hpp>
-#include<yade/pkg-dem/Clump.hpp>
+// #include<yade/pkg/dem/Shop.hpp>
+#include<yade/pkg/dem/Clump.hpp>
 
 // local copy
 #include<boost/math/nonfinite_num_facets.hpp>
@@ -55,8 +55,9 @@
 
 using namespace boost;
 using namespace std;
+namespace py = boost::python;
 
-#include<yade/lib-serialization/ObjectIO.hpp>
+#include<yade/lib/serialization/ObjectIO.hpp>
 
 #include<yade/extra/boost_python_len.hpp>
 
@@ -122,7 +123,6 @@ class pyBodyContainer{
 		shared_ptr<Body> clumpBody=shared_ptr<Body>(new Body());
 		shared_ptr<Clump> clump=shared_ptr<Clump>(new Clump());
 		clumpBody->shape=clump;
-		clumpBody->setDynamic(true);
 		clumpBody->setBounded(false);
 		proxee->insert(clumpBody);
 		// add clump members to the clump
@@ -199,6 +199,7 @@ class pyInteractionContainer{
 			//if(!PySequence_Check(id12.ptr())) throw invalid_argument("Key must be a tuple");
 			//if(python::len(id12)!=2) throw invalid_argument("Key must be a 2-tuple: id1,id2.");
 			if(id12.size()==2){
+				//if(max(id12[0],id12[1])>
 				shared_ptr<Interaction> i=proxee->find(id12[0],id12[1]);
 				if(i) return i; else { PyErr_SetString(PyExc_IndexError,"No such interaction"); python::throw_error_already_set(); /* make compiler happy; never reached */ return shared_ptr<Interaction>(); }
 			}
@@ -368,16 +369,15 @@ class pyOmega{
 	}
 	bool isRunning(){ return OMEGA.isRunning(); }
 	python::object get_filename(){ string f=OMEGA.sceneFile; if(f.size()>0) return python::object(f); return python::object();}
-	void load(std::string fileName) {
+	void load(std::string fileName,bool quiet=false) {
 		Py_BEGIN_ALLOW_THREADS; OMEGA.stop(); Py_END_ALLOW_THREADS; 
-		OMEGA.loadSimulation(fileName);
+		OMEGA.loadSimulation(fileName,quiet);
 		OMEGA.createSimulationLoop();
 		mapLabeledEntitiesToVariables();
-		LOG_DEBUG("LOAD!");
 	}
-	void reload(){	load(OMEGA.sceneFile);}
-	void saveTmp(string mark=""){ save(":memory:"+mark);}
-	void loadTmp(string mark=""){ load(":memory:"+mark);}
+	void reload(bool quiet=false){	load(OMEGA.sceneFile,quiet);}
+	void saveTmp(string mark="", bool quiet=false){ save(":memory:"+mark,quiet);}
+	void loadTmp(string mark="", bool quiet=false){ load(":memory:"+mark,quiet);}
 	python::list lsTmp(){ python::list ret; typedef pair<std::string,string> strstr; FOREACH(const strstr& sim,OMEGA.memSavedSimulations){ string mark=sim.first; boost::algorithm::replace_first(mark,":memory:",""); ret.append(mark); } return ret; }
 	void tmpToFile(string mark, string filename){
 		if(OMEGA.memSavedSimulations.count(":memory:"+mark)==0) throw runtime_error("No memory-saved simulation named "+mark);
@@ -399,11 +399,10 @@ class pyOmega{
 	void switchScene(){ std::swap(OMEGA.scene,OMEGA.sceneAnother); }
 	shared_ptr<Scene> scene_get(){ return OMEGA.getScene(); }
 
-	void save(std::string fileName){
+	void save(std::string fileName,bool quiet=false){
 		assertScene();
-		OMEGA.saveSimulation(fileName);
+		OMEGA.saveSimulation(fileName,quiet);
 		// OMEGA.sceneFile=fileName; // done in Omega::saveSimulation;
-		LOG_DEBUG("SAVE!");
 	}
 	
 	python::list miscParams_get(){
@@ -422,12 +421,18 @@ class pyOmega{
 		}
 	}
 
-	vector<shared_ptr<Engine> > engines_get(void){assertScene(); return OMEGA.getScene()->engines;}
-	void engines_set(const vector<shared_ptr<Engine> >& egs){assertScene(); OMEGA.getScene()->engines.clear(); FOREACH(const shared_ptr<Engine>& e, egs) OMEGA.getScene()->engines.push_back(e); mapLabeledEntitiesToVariables(); }
-	vector<shared_ptr<Engine> > initializers_get(void){assertScene(); return OMEGA.getScene()->initializers;}
-	void initializers_set(const vector<shared_ptr<Engine> >& egs){assertScene(); OMEGA.getScene()->initializers.clear(); FOREACH(const shared_ptr<Engine>& e, egs) OMEGA.getScene()->initializers.push_back(e); mapLabeledEntitiesToVariables(); OMEGA.getScene()->needsInitializers=true; }
 
-	
+	vector<shared_ptr<Engine> > engines_get(void){assertScene(); Scene* scene=OMEGA.getScene().get(); return scene->_nextEngines.empty()?scene->engines:scene->_nextEngines;}
+	void engines_set(const vector<shared_ptr<Engine> >& egs){
+		assertScene(); Scene* scene=OMEGA.getScene().get();
+		if(scene->subStep<0) scene->engines=egs; // not inside the engine loop right now, ok to update directly
+		else scene->_nextEngines=egs; // inside the engine loop, update _nextEngines; O.engines picks that up automatically, and Scene::moveToNextTimestep will put them in place of engines at the start of the next loop
+		mapLabeledEntitiesToVariables();
+	}
+	// raw access to engines/_nextEngines, for debugging
+	vector<shared_ptr<Engine> > currEngines_get(){ return OMEGA.getScene()->engines; }
+	vector<shared_ptr<Engine> > nextEngines_get(){ return OMEGA.getScene()->_nextEngines; }
+
 	pyBodyContainer bodies_get(void){assertScene(); return pyBodyContainer(OMEGA.getScene()->bodies); }
 	pyInteractionContainer interactions_get(void){assertScene(); return pyInteractionContainer(OMEGA.getScene()->interactions); }
 	
@@ -518,7 +523,7 @@ BOOST_PYTHON_MODULE(wrapper)
 
 	python::class_<pyOmega>("Omega")
 		.add_property("iter",&pyOmega::iter,"Get current step number")
-		.add_property("subStep",&pyOmega::subStep,"Get the current subStep number (only meaningful if O.subStepping==True)")
+		.add_property("subStep",&pyOmega::subStep,"Get the current subStep number (only meaningful if O.subStepping==True); -1 when outside the loop, otherwise either 0 (O.subStepping==False) or number of engine to be run (O.subStepping==True)")
 		.add_property("subStepping",&pyOmega::subStepping_get,&pyOmega::subStepping_set,"Get/set whether subStepping is active.")
 		.add_property("stopAtIter",&pyOmega::stopAtIter_get,&pyOmega::stopAtIter_set,"Get/set number of iteration after which the simulation will stop.")
 		.add_property("time",&pyOmega::time,"Return virtual (model world) time of the simulation.")
@@ -526,11 +531,11 @@ BOOST_PYTHON_MODULE(wrapper)
 		.add_property("dt",&pyOmega::dt_get,&pyOmega::dt_set,"Current timestep (Δt) value.\n\n* assigning negative value enables dynamic Δt (by looking for a :yref:`TimeStepper` in :yref:`O.engine<Omega.engines>`) and sets positive timestep ``O.dt=|Δt|`` (will be used until the timestepper is run and updates it)\n* assigning positive value sets Δt to that value and disables dynamic Δt (via :yref:`TimeStepper`, if there is one).\n\n:yref:`dynDt<Omega.dynDt>` can be used to query whether dynamic Δt is in use.")
 		.add_property("dynDt",&pyOmega::dynDt_get,"Whether a :yref:`TimeStepper` is used for dynamic Δt control. See :yref:`dt<Omega.dt>` on how to enable/disable :yref:`TimeStepper`.")
 		.add_property("dynDtAvailable",&pyOmega::dynDtAvailable_get,"Whether a :yref:`TimeStepper` is amongst :yref:`O.engines<Omega.engines>`, activated or not.")
-		.def("load",&pyOmega::load,"Load simulation from file.")
-		.def("reload",&pyOmega::reload,"Reload current simulation")
-		.def("save",&pyOmega::save,"Save current simulation to file (should be .xml or .xml.bz2)")
-		.def("loadTmp",&pyOmega::loadTmp,(python::args("mark")=""),"Load simulation previously stored in memory by saveTmp. *mark* optionally distinguishes multiple saved simulations")
-		.def("saveTmp",&pyOmega::saveTmp,(python::args("mark")=""),"Save simulation to memory (disappears at shutdown), can be loaded later with loadTmp. *mark* optionally distinguishes different memory-saved simulations.")
+		.def("load",&pyOmega::load,(py::arg("file"),py::arg("quiet")=false),"Load simulation from file.")
+		.def("reload",&pyOmega::reload,(py::arg("quiet")=false),"Reload current simulation")
+		.def("save",&pyOmega::save,(py::arg("file"),py::arg("quiet")=false),"Save current simulation to file (should be .xml or .xml.bz2)")
+		.def("loadTmp",&pyOmega::loadTmp,(py::arg("mark")="",py::arg("quiet")=false),"Load simulation previously stored in memory by saveTmp. *mark* optionally distinguishes multiple saved simulations")
+		.def("saveTmp",&pyOmega::saveTmp,(py::arg("mark")="",py::arg("quiet")=false),"Save simulation to memory (disappears at shutdown), can be loaded later with loadTmp. *mark* optionally distinguishes different memory-saved simulations.")
 		.def("lsTmp",&pyOmega::lsTmp,"Return list of all memory-saved simulations.")
 		.def("tmpToFile",&pyOmega::tmpToFile,(python::arg("fileName"),python::arg("mark")=""),"Save XML of :yref:`saveTmp<Omega.saveTmp>`'d simulation into *fileName*.")
 		.def("tmpToString",&pyOmega::tmpToString,(python::arg("mark")=""),"Return XML of :yref:`saveTmp<Omega.saveTmp>`'d simulation as string.")
@@ -548,8 +553,9 @@ BOOST_PYTHON_MODULE(wrapper)
 		.def("plugins",&pyOmega::plugins_get,"Return list of all plugins registered in the class factory.")
 		.def("_sceneObj",&pyOmega::scene_get,"Return the :yref:`scene <Scene>` object. Debugging only, all (or most) :yref:`Scene` functionality is proxies through :yref:`Omega`.")
 		.add_property("engines",&pyOmega::engines_get,&pyOmega::engines_set,"List of engines in the simulation (Scene::engines).")
+		.add_property("_currEngines",&pyOmega::currEngines_get,"Currently running engines; debugging only!")
+		.add_property("_nextEngines",&pyOmega::nextEngines_get,"Engines for the next step, if different from the current ones, otherwise empty; debugging only!")
 		.add_property("miscParams",&pyOmega::miscParams_get,&pyOmega::miscParams_set,"MiscParams in the simulation (Scene::mistParams), usually used to save serializables that don't fit anywhere else, like GL functors")
-		.add_property("initializers",&pyOmega::initializers_get,&pyOmega::initializers_set,"List of initializers (Scene::initializers).")
 		.add_property("bodies",&pyOmega::bodies_get,"Bodies in the current simulation (container supporting index access by id and iteration)")
 		.add_property("interactions",&pyOmega::interactions_get,"Interactions in the current simulation (container supporting index acces by either (id1,id2) or interactionNumber and iteration)")
 		.add_property("materials",&pyOmega::materials_get,"Shared materials; they can be accessed by id or by label")
@@ -599,7 +605,7 @@ BOOST_PYTHON_MODULE(wrapper)
 		.def("eraseNonReal",&pyInteractionContainer::eraseNonReal,"Erase all interactions that are not :yref:`real <InteractionContainer.isReal>`.")
 		.def("erase",&pyInteractionContainer::erase,"Erase one interaction, given by id1, id2 (internally, ``requestErase`` is called -- the interaction might still exist as potential, if the :yref:`Collider` decides so).")
 		.add_property("serializeSorted",&pyInteractionContainer::serializeSorted_get,&pyInteractionContainer::serializeSorted_set)
-		.def("clear",&pyInteractionContainer::clear,"Remove all interactions");
+		.def("clear",&pyInteractionContainer::clear,"Remove all interactions, and invalidate persistent collider data (if the collider supports it).");
 	python::class_<pyInteractionIterator>("InteractionIterator",python::init<pyInteractionIterator&>())
 		.def("__iter__",&pyInteractionIterator::pyIter)
 		.def("next",&pyInteractionIterator::pyNext);
