@@ -9,6 +9,7 @@ sp.toSimulation()
 O.bodies.append(utils.wall(position=0,axis=2))
 
 O.engines=[
+	#SubdomainBalancer(),
 	ForceResetter(),
 	InsertionSortCollider([Bo1_Sphere_Aabb(),Bo1_Wall_Aabb()]),
 	InteractionLoop(
@@ -21,4 +22,5 @@ O.engines=[
 ]
 O.dt=.7*utils.PWaveTimeStep()
 O.saveTmp()
+O.step()
 
