@@ -45,7 +45,7 @@ O.engines=[
 ]
 
 O.periodic=True
-O.cell.refSize=Vector3(1,1,1)
+O.cell.refSize=(1,1,1)
 
 O.dt=.01*tc
 
