@@ -1,5 +1,5 @@
 O.periodic=True
-O.cell.refSize=Vector3(.55,.55,.55)
+O.cell.refSize=(.55,.55,.55)
 O.bodies.append(utils.facet([[.4,.0001,.3],[.2,.0001,.3],[.3,.2,.2]]))
 O.bodies.append(utils.sphere([.3,.1,.4],.05,dynamic=True))
 O.bodies.append(utils.sphere([.200001,.2000001,.4],.05,dynamic=False))
